import logging
import mimetypes
from sorl.thumbnail import get_thumbnail
from django_cradmin.imageutils.backends import backendinterface


logger = logging.getLogger(__name__)


class SorlThumbnail(backendinterface.Interface):
    """
    Sorl-thumbnail backend for ``django_cradmin.imageutils``.
    """

    def transform_image(self, imageurl, **options):
        width = options.get('width', None)
        height = options.get('height', None)
        quality = options.get('quality', 100)
        crop = options.get('crop', 'limit')

        if not width and not height:
            raise ValueError('At least one of width and height must be specified')
        width = width or ''
        height = height or ''
        size = '{}x{}'.format(width, height)

        if crop == 'limit':
            upscale = False
            sorl_crop = None
        else:
            upscale = True
            sorl_crop = 'center'

        mimetype = mimetypes.guess_type(imageurl)[0]
        if mimetype == 'image/jpeg':
            imageformat = 'JPEG'
        elif mimetype == 'image/png':
            imageformat = 'PNG'
        elif mimetype == 'image/gif':
            imageformat = 'GIF'
        else:
            imageformat = 'JPEG'

        try:
            thumbnail = get_thumbnail(
                imageurl, size,
                upscale=upscale,
                quality=quality,
                crop=sorl_crop,
                format=imageformat)
            return thumbnail.url
        except Exception as e:
            logger.exception(e)
            return ''



from __future__ import unicode_literals

from future import standard_library

standard_library.install_aliases()
import urllib.request
import urllib.parse
import urllib.error
from django.utils.translation import ugettext_lazy as _
from django.forms import widgets
from django.template.loader import render_to_string


class ModelChoiceWidget(widgets.TextInput):
    """
    Model choice widget that uses an iframe popup to enable users
    to select their foreign key value.
    """
    #: The template used to render the widget.
    template_name = 'django_cradmin/widgets/modelchoice.django.html'

    #: Do not override this (if you set this to hidden, the widget is not rendered correctly).
    input_type = 'text'

    #: Set this to ``True`` to debug the value of the input field.
    #: Setting this to ``True``, makes the ``type`` of the actually
    #: submitted input field to ``text`` instead of ``hidden``.
    #: Probably only useful for debugging.
    input_field_visible = False

    #: The default select-button text. You can override this in a subclass,
    #: or use the ``selectbutton_text``-argument for the constructor to
    #: change the button text.
    default_selectbutton_text = _('Select ...')

    def __init__(self, queryset, selectview_url, preview='',
                 selectbutton_text=None):
        self.queryset = queryset
        self.preview = preview
        self.selectview_url = selectview_url
        self.selectbutton_text = selectbutton_text or self.default_selectbutton_text
        super(ModelChoiceWidget, self).__init__()

    def __make_selectview_url(self, fieldid, current_value):
        return '{}?{}'.format(
            self.selectview_url, urllib.parse.urlencode({
                'foreignkey_select_current_value': current_value,
                'foreignkey_select_fieldid': fieldid,
            }))

    def get_rendered_input_type(self):
        if self.input_field_visible:
            return 'text'
        else:
            return 'hidden'

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        fieldid = attrs['id']
        return render_to_string(self.template_name, {
            'preview': self.preview,
            'fieldname': name,
            'fieldid': fieldid,
            'fieldvalue': value,
            'selectview_url': self.__make_selectview_url(fieldid, value),
            'selectbutton_text': self.selectbutton_text,
            'input_type': self.input_type,
            'rendered_input_type': self.get_rendered_input_type()
        })



from django.views.generic import TemplateView

from django_cradmin.viewhelpers import listbuilder


class ItemFrameDjangoApp(listbuilder.itemframe.Link):
    valuealias = 'djangoappconfig'

    def get_url(self):
        return self.djangoappconfig.get_indexview_url()


class ItemValueDjangoApp(listbuilder.itemvalue.FocusBox):
    valuealias = 'djangoappconfig'
    template_name = 'django_cradmin/superuserui/listbuilder/itemvalue-djangoapp.django.html'


class View(TemplateView):
    template_name = 'django_cradmin/superuserui/dashboard.django.html'

    def get_djangoapps_listbuilder_list(self):
        return listbuilder.lists.RowList.from_value_iterable(
            value_iterable=self.request.cradmin_instance.get_superuserui_registry().iter_djangoappconfigs(),
            frame_renderer_class=ItemFrameDjangoApp,
            value_renderer_class=ItemValueDjangoApp
        )

    def get_context_data(self, **kwargs):
        context = super(View, self).get_context_data(**kwargs)
        context['djangoapps_list'] = self.get_djangoapps_listbuilder_list()
        context['pagetitle'] = self.request.cradmin_instance.get_superuserui_registry().get_title()
        return context



from django.conf import settings


def get_setting(settingvariable, fallback=None):
    return getattr(settings, settingvariable, fallback)


def get_django_cradmin_menu_scroll_top_fixed_setting():
    scroll_top_fixed_settings = get_setting('DJANGO_CRADMIN_MENU_SCROLL_TOP_FIXED', False)
    if scroll_top_fixed_settings is True:
        return {
            'cssClasses': {
                'scrollingClass': 'django-cradmin-menu-scrolling',
            }
        }
    else:
        return False


def cradmin(request):
    return {
        'DJANGO_CRADMIN_THEME_PATH': get_setting(
            'DJANGO_CRADMIN_THEME_PATH',
            'django_cradmin/dist/css/cradmin_theme_default/theme.css'),
        'DJANGO_CRADMIN_MENU_SCROLL_TOP_FIXED': get_django_cradmin_menu_scroll_top_fixed_setting(),
        'DJANGO_CRADMIN_MOMENTJS_LOCALE': get_setting(
            'DJANGO_CRADMIN_MOMENTJS_LOCALE', None),
        'DJANGO_CRADMIN_CSS_ICON_LIBRARY_PATH': get_setting(
            'DJANGO_CRADMIN_CSS_ICON_LIBRARY_PATH',
            'django_cradmin/dist/vendor/fonts/fontawesome/css/font-awesome.min.css'),
        'DJANGO_CRADMIN_HIDE_PAGE_HEADER': get_setting(
            'DJANGO_CRADMIN_HIDE_PAGE_HEADER', False),
    }



from django.http import QueryDict
from django.test import TestCase
from future.standard_library import install_aliases

from django_cradmin import urlutils

install_aliases()

from urllib.parse import urlsplit


class TestUpdateQuerystring(TestCase):
    def test_simple(self):
        self.assertEqual(
            urlutils.update_querystring('http://example.com',
                                        {'next': 'http://example.com/admin'}),
            'http://example.com?next=http%3A%2F%2Fexample.com%2Fadmin'
        )

    def test_update_existing(self):
        self.assertEqual(
            urlutils.update_querystring('http://example.com?next=/test',
                                        {'next': '/updated'}),
            'http://example.com?next=%2Fupdated'
        )

    def test_keep_existing(self):
        url = urlutils.update_querystring('http://example.com?next=/test',
                                          {'other': 'otherstuff'})
        url_split = urlsplit(url)
        querydict = QueryDict(url_split.query)
        self.assertEqual(querydict.dict(), {
            'next': '/test',
            'other': 'otherstuff',
        })

    def test_ignore_none_values(self):
        self.assertEqual(
            urlutils.update_querystring('http://example.com',
                                        {'next': None}),
            'http://example.com'
        )

    def test_do_not_ignore_none_values(self):
        self.assertEqual(
            urlutils.update_querystring('http://example.com',
                                        {'next': None}, ignore_none_values=False),
            'http://example.com?next=None'
        )



from __future__ import unicode_literals

import json
from xml.sax.saxutils import quoteattr

from django.utils.translation import pgettext_lazy

from django_cradmin.viewhelpers import listbuilder
from django_cradmin.viewhelpers.multiselect2 import selected_item_renderer
from django_cradmin.viewhelpers.multiselect2 import target_renderer
from django_cradmin.viewhelpers.multiselect2 import widget_preview_renderer


class ItemValue(listbuilder.itemvalue.TitleDescription):
    """
    Listbuilder itemvalue renderable that renders some information
    and a "Select" buttonn that works out of the box with
    :class:`django_cradmin.viewhelpers.multiselect2.target_renderer.Target`
    to enable multiselect.

    If you have multiple lists with multiselect2 on the same page,
    you have to ensure the DOM IDs are unique by overriding:

    - :meth:`~.ItemValue.get_selectbutton_dom_id`.
    - :meth:`~.ItemValue.get_target_dom_id` (or use the ``target_dom_id`` parameter). Make sure you
      set :meth:`django_cradmin.viewhelpers.multiselect2.target_renderer.Target.get_target_dom_id`
      on your corresponding Target renderer to reflect the new ID.
    - :meth:`django_cradmin.viewhelpers.multiselect2.selected_item_renderer.SelectedItem.get_inputfield_name`
      of the :meth:`~.ItemValue.get_selected_item_renderer_class`.

    See :doc:`viewhelpers_listbuilder` for more information about listbuilder.
    """

    #: The template used to render this renderable.
    template_name = 'django_cradmin/viewhelpers/multiselect2/listbuilder_itemvalues/itemvalue.django.html'

    #: Selected item rendrerer class.
    selected_item_renderer_class = selected_item_renderer.SelectedItem

    def __init__(self, *args, **kwargs):
        """
        Args:
            target_dom_id: See :meth:`~.ItemValue.get_target_dom_id`.
            is_selected: Mark the item as selected on load.
        """
        self.target_dom_id = kwargs.pop('target_dom_id', None)
        super(ItemValue, self).__init__(*args, **kwargs)
        self.selected_item_renderer = self.make_selected_item_renderer()

    def get_target_dom_id(self):
        """
        Returns:
            str: The ``dom_id`` of the :class:`django_cradmin.viewhelpers.multiselect2.target_renderer.Target`.

            Defaults to the ``target_dom_id`` parameter falling back on
            :obj:`django_cradmin.viewhelpers.multiselect2.target_renderer.Target.default_target_dom_id`.
        """
        if self.target_dom_id:
            return self.target_dom_id
        else:
            return target_renderer.Target.default_target_dom_id

    def get_selectbutton_dom_id(self):
        """
        Returns:
            str: Get the DOM id of the select button.

            Defaults to ``django_cradmin_multiselect2_selectbutton_<self.value.pk>``.

            If you have multiple lists with multiselect2 on the same page, you must
            make sure this is unique.
        """
        return 'django_cradmin_multiselect2_selectbutton_{}'.format(self.value.pk)

    def get_base_css_classes_list(self):
        """
        Adds the ``django-cradmin-multiselect2-itemvalue`` css class
        in addition to the classes added by the superclasses.
        """
        css_classes = super(ItemValue, self).get_base_css_classes_list()
        css_classes.append('django-cradmin-multiselect2-itemvalue')
        return css_classes

    def get_selectbutton_text(self):
        """
        Returns:
            str: The text for the select button.

            Defaults to ``"Select"`` (translatable).
        """
        return pgettext_lazy('multiselect2 select button', 'Select')

    def get_selectbutton_aria_label(self):
        """
        Returns:
            str: The text for the aria-label of the select button.

            Defaults to ``"Select <self.get_title()>"`` (translatable).
        """
        return pgettext_lazy('multiselect2 select button', 'Select "%(title)s"') % {
            'title': self.get_title()
        }

    def get_custom_data(self):
        """
        Returns:
            dict: Custom data for the ``custom_data``-attribute for the angularjs directive.

            Used by :meth:`.get_select_directive_dict`.
        """
        return None

    def get_select_directive_dict(self, request):
        """
        Get options for the ``django-cradmin-multiselect2-select`` angularjs
        directive.

        Args:
            request: A Django HttpRequest object.

        Returns:
            dict: With options for the directive.
        """
        return {
            'preview_container_css_selector': '.django-cradmin-multiselect2-itemvalue',
            'preview_css_selector': '.django-cradmin-multiselect2-selected-item',
            'item_wrapper_css_selector': 'li',
            'target_dom_id': self.get_target_dom_id(),
            'custom_data': self.get_custom_data(),
            'is_selected': self.kwargs.get('is_selected', False),
        }

    def get_select_directive_json(self, request):
        """
        Args:
            request: A Django HttpRequest object.

        Returns:
            str: The return value of :meth:`.get_select_directive_dict`
            as a json encoded and xml attribute encoded string.
        """
        return quoteattr(json.dumps(self.get_select_directive_dict(request=request)))

    def get_selected_item_renderer_class(self):
        """
        Returns:
            SelectedItem class: The renderable class to use
            to render the selected item in the corresponding
            :class:`django_cradmin.viewhelpers.multiselect2.target_renderer.Target`.

            Must be :class:`django_cradmin.viewhelpers.multiselect2.selected_item_renderer.SelectedItem`
            or a subclass.

            Defaults to :obj:`.selected_item_renderer_class`.
        """
        return self.selected_item_renderer_class

    def make_selected_item_renderer(self):
        """
        Returns:
            django_cradmin.viewhelpers.multiselect2.selected_item_renderer.SelectedItem: An instance
            of :meth:`.get_selected_item_renderer_class`.

            This is called in ``__init__`` and the return value is stored in ``self.selected_item_renderer``.
        """
        return self.get_selected_item_renderer_class()(value=self.value)

    def get_context_data(self, request=None):
        context = super(ItemValue, self).get_context_data(request=request)
        context['select_directive_json'] = self.get_select_directive_json(request=request)
        return context


class ManyToManySelect(ItemValue):
    """
    Many-to-many select item value.

    Extends :class:`.ItemValue` with the data needed for
    :class:`django_cradmin.viewhelpers.multiselect2.manytomanywidget.Widget`.
    """
    def get_manytomanyfield_preview_renderer_class(self):
        """
        Returns:
            django_cradmin.viewhelpers.multiselect2.widget_preview_renderer.Value: The class
            used to render the preview html (see :meth:`.get_manytomanyfield_preview_html`).
        """
        return widget_preview_renderer.Value

    def get_manytomanyfield_preview_html(self):
        """
        Returns:
            str: Preview HTML added as the ``preview`` key in :meth:`.get_custom_data`.
            This is added to the preview list rendered by
            :class:`django_cradmin.viewhelpers.multiselect2.manytomanywidget.Widget`
            when the selection is confirmed.
        """
        renderer_class = self.get_manytomanyfield_preview_renderer_class()
        return renderer_class(value=self.value, wrap_in_li_element=True).render()

    def get_custom_data(self):
        """
        Returns:
            dict: A dict with:

            - ``value``: Taken from the ``get_inputfield_value`` method
              of :meth:`.ItemValue.make_selected_item_renderer`. See
              :meth:`~django_cradmin.viewhelpers.multiselect2.selected_item_renderer.SelectedItem.get_inputfield_value`.
            - ``preview``: Preview HTML returned by :meth:`.get_manytomanyfield_preview_html`.
        """
        return {
            'value': self.selected_item_renderer.get_inputfield_value(),
            'preview': self.get_manytomanyfield_preview_html()
        }



from __future__ import unicode_literals
from builtins import str
from django.contrib import messages
from django.views.generic import DeleteView as DjangoDeleteView
from django.utils.translation import ugettext_lazy as _
from django_cradmin.viewhelpers.mixins import QuerysetForRoleMixin


class DeleteView(QuerysetForRoleMixin, DjangoDeleteView):

    #: The name of the template to use.
    template_name = 'django_cradmin/viewhelpers/delete.django.html'

    def get_pagetitle(self):
        """
        Get the page title (the title tag).

        Defaults to ``Delete <verbose_name model>``.
        """
        return _('Delete %(what)s') % {'what': self.get_object_preview()}

    def get_action_label(self):
        """
        The action we are performing.

        Used as the prefix of the page title (see :meth:`.get_pagetitle`),
        and as the default for :meth:`.get_delete_button_label`.
        """
        return _('Delete')

    def get_delete_button_label(self):
        """
        The label of the delete button.

        Defaults to :meth:`.get_action_label`.
        """
        return self.get_action_label()

    def get_object_preview(self):
        """
        The preview of the object. Used when asking the user if he/she wants to
        delete the current object.
        """
        obj = self.get_object()
        return str(obj)

    def get_confirm_message(self):
        """
        Get the confirm message shown in the focus area of the view.
        """
        return _('Are you sure you want to delete "%(object_preview)s"?') % {
            'object_preview': self.get_object_preview()
        }

    def get_success_url(self):
        """
        Get the URL to go to if object was deleted.

        Defaults to the INDEX view of the current app.
        """
        return self.request.cradmin_app.reverse_appindexurl()

    def get_context_data(self, **kwargs):
        context = super(DeleteView, self).get_context_data(**kwargs)
        obj = context['object']
        context['model_verbose_name'] = obj._meta.verbose_name
        context['success_url'] = self.get_success_url()
        context['object_preview'] = self.get_object_preview()
        context['pagetitle'] = self.get_pagetitle()
        context['confirm_message'] = self.get_confirm_message()
        context['delete_button_label'] = self.get_delete_button_label()
        return context

    def get_success_message(self, object_preview):
        """
        Override this to provide a success message.

        Used by :meth:`.add_success_messages`.
        """
        return _('Deleted "%(what)s"') % {
            'what': object_preview
        }

    def add_success_messages(self, object_preview):
        """
        Called after the object has been deleted.

        Defaults to add :meth:`.get_success_message` as a django messages
        success message if :meth:`.get_success_message` returns anything.

        You can override this to add multiple messages or to show messages in some other way.
        """
        success_message = self.get_success_message(object_preview)
        if success_message:
            messages.success(self.request, success_message)

    def delete(self, request, *args, **kwargs):
        object_preview = self.get_object_preview()
        response = super(DeleteView, self).delete(request, *args, **kwargs)
        self.add_success_messages(object_preview)
        return response



from __future__ import unicode_literals
from django.contrib import admin
from django.contrib.humanize.templatetags.humanize import naturaltime
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata


class GenericTokenWithMetadataAdmin(admin.ModelAdmin):
    list_display = (
        'token',
        'content_object',
        'app',
        'created_datetime',
        'expiration_datetime',
        'expiration_naturaltime',
    )
    search_fields = (
        'app',
        'user__email',
        'token',
    )
    list_filter = (
        'app',
        'created_datetime',
        'expiration_datetime',
    )
    readonly_fields = (
        'token',
        'content_type',
        'single_use',
        'metadata_json',
        'object_id',
        'app',
        'created_datetime',
    )
    ordering = ['-created_datetime']

    def get_queryset(self, request):
        if request.user.is_superuser:
            return super(GenericTokenWithMetadataAdmin, self).get_queryset(request)
        else:
            return GenericTokenWithMetadata.objects.none()

    def expiration_naturaltime(self, obj):
        return naturaltime(obj.expiration_datetime)
    expiration_naturaltime.short_description = 'Expires'
    expiration_naturaltime.admin_order_field = 'expiration_datetime'

admin.site.register(GenericTokenWithMetadata, GenericTokenWithMetadataAdmin)



from django.template import RequestContext
from django.test import TestCase, RequestFactory

from django_cradmin.renderable import AbstractRenderable


class MinimalRenderable(AbstractRenderable):
    template_name = 'cradmin_viewhelpers_testapp/listbuilder/minimal-renderable.django.html'


class TestAbstractRenderable(TestCase):
    def test_get_template_name_no_template_name(self):
        with self.assertRaises(NotImplementedError):
            AbstractRenderable().get_template_name()

    def test_get_template_name_has_template_name(self):
        MinimalRenderable().get_template_name()  # NotImplementedError not raised

    def test_get_context_data(self):
        renderable = AbstractRenderable()
        self.assertEqual({'me': renderable}, renderable.get_context_data())

    def test_get_template_context_object_without_request(self):
        renderable = AbstractRenderable()
        self.assertEqual({'me': renderable}, renderable.get_template_context_object())

    def test_get_template_context_object_with_request(self):
        renderable = AbstractRenderable()
        request = RequestFactory().get('')
        context_object = renderable.get_template_context_object(request)
        self.assertTrue(isinstance(context_object, RequestContext))
        self.assertEqual(renderable, context_object['me'])

    def test_render(self):
        self.assertEqual('Test', MinimalRenderable().render().strip())



from __future__ import unicode_literals
from django.template import Library
from django_cradmin import imageutils
import logging
from django_cradmin.imageutils.backends.backendinterface import ImageTypeMapSettingNotDefined, InvalidImageType

register = Library()
logger = logging.getLogger(__name__)


@register.simple_tag(takes_context=True)
def cradmin_transform_image(context, imageurl, **options):
    """
    Tag wrapper around
    :meth:`~django_cradmin.imageutils.backends.backendinterface.Interface.transform_image`.

    Parameters:

        imageurl: The URL of the image to transform.
        options: Image transformation options.
    """
    request = context['request']
    imageurl = request.build_absolute_uri(imageurl)
    if 'options' in options:
        options = options['options']
    return imageutils.get_backend().transform_image(imageurl, **options)


@register.simple_tag(takes_context=True)
def cradmin_transform_image_using_imagetype(context, imageurl, imagetype, fallbackoptions=None):
    """
    Tag wrapper around
    :meth:`~django_cradmin.imageutils.backends.backendinterface.Interface.transform_image_using_imagetype`.

    Parameters:

        imageurl: The URL of the image to transform.
        imagetype: See
            :meth:`~django_cradmin.imageutils.backends.backendinterface.Interface.transform_image_using_imagetype`.
        fallbackoptions: An optional dict of options to use if ``imagetype``
            is not in the :setting:`DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP` setting.
    """
    request = context['request']
    imageurl = request.build_absolute_uri(imageurl)
    try:
        return imageutils.get_backend().transform_image_using_imagetype(imageurl, imagetype)
    except (ImageTypeMapSettingNotDefined, InvalidImageType):
        if fallbackoptions:
            return imageutils.get_backend().transform_image(imageurl, **fallbackoptions)
        else:
            raise


@register.inclusion_tag('django_cradmin/imageutils/templatetags/archiveimage-tag.django.html',
                        takes_context=True)
def cradmin_create_archiveimage_tag(context, archiveimage, imagetype, css_class='', fallbackoptions=None):
    """
    Creates an ``<img>`` tag from the given cradmin archiveimage.

    The URL is generated using
    :meth:`.cradmin_transform_image_using_imagetype`.

    The alt-tag is generated from ``archiveimage.screenreader_text``.
    """
    return {
        'archiveimage': archiveimage,
        'url': cradmin_transform_image_using_imagetype(
            context=context,
            imageurl=archiveimage.image.url,
            imagetype=imagetype,
            fallbackoptions=fallbackoptions),
        'css_class': css_class,
    }



from django_cradmin.viewhelpers import listbuilder


class List(listbuilder.lists.RowList):
    """
    Renders a list for previewing selection in
    :class:`django_cradmin.viewhelpers.multiselect2.manytomanywidget.Widget`.
    """
    template_name = 'django_cradmin/viewhelpers/multiselect2/manytomanywidget/preview-list.django.html'

    @classmethod
    def from_value_iterable(cls, value_iterable,
                            value_renderer_class=None,
                            frame_renderer_class=None,
                            **listkwargs):
        """
        Overrides :meth:`.django_cradmin.viewhelpers.listbuilder.base.List.from_value_iterable`
        to set :class:`.Value` as the default ``value_renderer_class``.
        """
        if value_renderer_class is None:
            value_renderer_class = Value
        return super(List, cls).from_value_iterable(
            value_iterable=value_iterable,
            value_renderer_class=value_renderer_class,
            frame_renderer_class=frame_renderer_class,
            **listkwargs)

    def get_base_css_classes_list(self):
        """
        Adds the ``django-cradmin-multiselect2-preview-list`` css class
        to the css classes added by the superclasses.
        """
        css_classes = super(List, self).get_base_css_classes_list()
        css_classes.append('django-cradmin-multiselect2-preview-list')
        return css_classes


class Value(listbuilder.base.ItemValueRenderer):
    """
    Renders a single value in :class:`.List` - also used
    by the ``get_manytomanyfield_preview_renderer_class`` method in
    :class:`~django_cradmin.viewhelpers.multiselect2.listbuilder_itemvalues.ManyToManySelect`
    """
    template_name = 'django_cradmin/viewhelpers/multiselect2/manytomanywidget/preview-value.django.html'

    def __init__(self, value, wrap_in_li_element=False):
        """
        Args:
            value: See :class:`django_cradmin.viewhelpers.listbuilder.base.AbstractItemRenderer`.
            wrap_in_li_element: If this is set to ``True``, we wrap the rendered item in
                an ``<li>``.

                This is set to ``True`` by the ``get_manytomanyfield_preview_html``-method in
                :class:`django_cradmin.viewhelpers.multiselect2.listbuilder_itemvalues.ManyToManySelect`
                to render items that can be dynamically added to a :class:`.List`.
        """
        self.wrap_in_li_element = wrap_in_li_element
        super(Value, self).__init__(value=value)

    def get_base_css_classes_list(self):
        """
        Adds the ``django-cradmin-multiselect2-preview-list-value`` css class
        to the css classes added by the superclasses.
        """
        css_classes = super(Value, self).get_base_css_classes_list()
        css_classes.append('django-cradmin-multiselect2-preview-list-value')
        return css_classes



from __future__ import unicode_literals
from builtins import object

from django.conf import settings

from django_cradmin.apps.cradmin_email import emailutils
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata, \
    get_expiration_datetime_for_app


class InviteEmail(emailutils.AbstractEmail):
    """
    The default invite email class for :class:`.InviteUrl`.

    You may want to provide your own and override
    :meth:`.InviteUrl.get_invite_email_class`.
    """
    subject_template = 'cradmin_invite/email/subject.django.txt'
    html_message_template = 'cradmin_invite/email/html_message.django.html'

    def get_context_data(self):
        context = super(InviteEmail, self).get_context_data()
        context.update({
            'DJANGO_CRADMIN_SITENAME': settings.DJANGO_CRADMIN_SITENAME
        })
        return context


class InviteUrl(object):
    """
    Sends an email with a link that the user clicks to accept an invite.

    Example:
        ::

            from django_cradmin.apps.cradmin_invite.utils import InviteUrl

            class InviteUrl(InviteUrl):
                def get_appname(self):
                    return 'myapp'

                def get_confirm_invite_url(self, generictoken):
                    return reverse('myapp-confirm-invite', kwargs={
                        'token': generictoken.token
                    })

            def myview(request):
                InviteUrl(request=request, private=True, content_object=someobject).send_email(
                    'test@example.com', 'test2@example.com')
                # ... or ...
                share_url = InviteUrl(request=request, private=False, content_object=someobject).get_share_url()
                # ... or ...
                InviteUrl(request=request, private=False, content_object=someobject).send_email(
                    'test@example.com', 'test2@example.com', 'test3@example.com')
    """

    def __init__(self, request, private, content_object, metadata=None, **kwargs):
        """

        Parameters:
            request: A Django HttpRequest object.
            private: If this is ``True`` we send unique single-use invite URLs.
            metadata: Metadata to accociate with the invite.

        """
        self.request = request
        self.private = private
        self.content_object = content_object
        self.metadata = metadata
        if 'expiration_datetime' in kwargs:
            self.expiration_datetime = kwargs['expiration_datetime']

    def get_appname(self):
        """
        Get the appname for
        :obj:`django_cradmin.apps.cradmin_generic_token_with_metadata.models.GenericTokenWithMetadata.app`.

        You must override this in subclasses.
        """
        raise NotImplementedError()

    def get_confirm_invite_url(self, generictoken):
        """
        Get the confirm invite view URL.

        Must be implemented in subclasses.

        Parameters:
            generictoken: A
                :class:`~django_cradmin.apps.cradmin_generic_token_with_metadata.models.GenericTokenWithMetadata`
                object.
        """
        raise NotImplementedError()

    def __get_absolute_confirm_invite_url(self, generictoken):
        return self.request.build_absolute_uri(self.get_confirm_invite_url(generictoken))

    def get_from_email(self):
        """
        Get the email sender address.

        Defaults to the ``DJANGO_CRADMIN_INVITE_FROM_EMAIL`` setting
        falling back on the ``DEFAULT_FROM_EMAIL`` setting.
        """
        return getattr(settings, 'DJANGO_CRADMIN_INVITE_FROM_EMAIL', settings.DEFAULT_FROM_EMAIL)

    def get_expiration_datetime(self):
        """
        Get the value to use for the ``expiration_datetime`` attribute of
        :class:`~django_cradmin.apps.cradmin_generic_token_with_metadata.models.GenericTokenWithMetadata`.

        Defaults to the expiration_datetime provided via the constructor,
        and falls back to getting the configured expiration datetime for
        the app.
        """
        if hasattr(self, 'expiration_datetime'):
            return self.expiration_datetime
        else:
            return get_expiration_datetime_for_app(self.get_appname())

    def _generate_generictoken(self, email=None):
        metadata = self.metadata
        if email:
            if self.metadata is None:
                metadata = {'email': email}
            elif isinstance(self.metadata, dict):
                if 'email' not in self.metadata:
                    metadata = self.metadata.copy()
                    metadata['email'] = email

        return GenericTokenWithMetadata.objects.generate(
            app=self.get_appname(),
            expiration_datetime=self.get_expiration_datetime(),
            content_object=self.content_object,
            metadata=metadata
        )

    def generate_generictoken(self, email=None):
        if self.private:
            # If private generate unique tokens
            return self._generate_generictoken(email=email)
        else:
            # If public, re-use the same token and ignore email argument
            if hasattr(self, '_generictoken'):
                return self._generictoken
            else:
                self._generictoken = self._generate_generictoken()
                return self._generictoken

    def get_invite_email_class(self):
        """
        Must return a subclass of :class:`django_cradmin.apps.cradmin_email.emailutils.AbstractEmail`.

        Defaults to :class:`.InviteEmail`.
        """
        return InviteEmail

    def get_extra_invite_email_context_data(self, generictoken):
        """
        Override this to provide extra context data for the
        :meth:`.get_invite_email_class`.

        Make sure you call ``super`` and extend the returned dict.
        """
        return {
            'confirm_invite_url': self.__get_absolute_confirm_invite_url(generictoken)
        }

    def send_email(self, *emails):
        """
        Generate a token and send an email containing an absolute invite
        URL for that token.

        If this InviteUrl is private, we generate a new token each
        email recipient, and if it is public, we re-use the same
        token.

        Private tokens are generated as single use tokens, and public tokens
        are unlimited use tokens.

        Private tokens gets the email automatically added to the metadata
        if metadata is a dict or None.
        """
        # TODO: Re-use email connection
        invite_email_class = self.get_invite_email_class()
        for email in emails:
            generictoken = self.generate_generictoken(email)
            invite_email_class(
                recipient=email,
                from_email=self.get_from_email(),
                extra_context_data=self.get_extra_invite_email_context_data(generictoken)
            ).send()

    def get_share_url(self):
        """
        Generate a token and return an absolute invite URL for that token.

        If this InviteUrl is private, we generate a new token each
        time this is called, and if it is public, we re-use the same
        token.

        Private tokens are generated as single use tokens, and public tokens
        are unlimited use tokens.
        """
        generictoken = self.generate_generictoken()
        return self.__get_absolute_confirm_invite_url(generictoken)

        # def get_tokens_stored_for_app(self):
        #     return GenericTokenWithMetadata.objects.filter_not_expired().filter(app=self.get_appname())



from __future__ import unicode_literals
from datetime import timedelta, datetime
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test import TestCase, RequestFactory
from django.utils import timezone
import htmls
from django_cradmin.python2_compatibility import mock
from django_cradmin.apps.cradmin_resetpassword.views.reset import ResetPasswordView
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata
from django_cradmin.tests.helpers import create_user


class TestResetPasswordView(TestCase):
    def setUp(self):
        self.testuser = create_user('testuser')

    def __get_url(self, token):
        return reverse('cradmin-resetpassword-reset', kwargs={'token': token})

    def _create_generic_token_with_metadata(self, user, created_datetime=None, expiration_datetime=None, **kwargs):
        generic_token_with_metadata = GenericTokenWithMetadata.objects.create(
            created_datetime=(created_datetime or timezone.now()),
            expiration_datetime=(expiration_datetime or (timezone.now() + timedelta(days=2))),
            content_object=user,
            app='cradmin_resetpassword',
            **kwargs)
        return generic_token_with_metadata

    def test_get(self):
        self._create_generic_token_with_metadata(token='valid-token', user=self.testuser)
        response = self.client.get(self.__get_url('valid-token'))
        selector = htmls.S(response.content)
        self.assertEqual(
            selector.one('h1').alltext_normalized,
            'Reset your password testuser')
        self.assertTrue(selector.exists('form#django_cradmin_resetpassword_reset_form'))
        self.assertTrue(selector.exists('input[type="password"][name="password1"]'))
        self.assertTrue(selector.exists('input[type="password"][name="password2"]'))

    def test_get_expired_token(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser,
            expiration_datetime=datetime(2014, 1, 1))
        response = self.client.get(self.__get_url('valid-token'))
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists('form#django_cradmin_resetpassword_reset_form'))
        self.assertTrue(selector.exists('#django_cradmin_resetpassword_reset_expired_message'))
        self.assertEqual(
            selector.one('#django_cradmin_resetpassword_reset_expired_message').alltext_normalized,
            'This password reset link has expired.')

    def test_get_invalid_token(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser)
        response = self.client.get(self.__get_url('invalid-token'))
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists('form#django_cradmin_resetpassword_reset_form'))
        self.assertTrue(selector.exists('#django_cradmin_resetpassword_reset_invalid_token_message'))
        self.assertEqual(
            selector.one('#django_cradmin_resetpassword_reset_invalid_token_message').alltext_normalized,
            'Invalid password reset URL. Are you sure you copied the entire URL from the email?')

    def test_post_passwords_not_matching(self):
        self._create_generic_token_with_metadata(token='valid-token', user=self.testuser)
        response = self.client.post(self.__get_url('valid-token'), {
            'password1': 'passwordOne',
            'password2': 'passwordTwo',
        })
        selector = htmls.S(response.content)
        self.assertIn(
            'The passwords do not match',
            selector.one('form#django_cradmin_resetpassword_reset_form').alltext_normalized)

    def test_post_expired_token(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser,
            expiration_datetime=datetime(2014, 1, 1))
        response = self.client.post(self.__get_url('valid-token'), {
            'password1': 'secret',
            'password2': 'secret',
        })
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists('form#django_cradmin_resetpassword_reset_form'))
        self.assertTrue(selector.exists('#django_cradmin_resetpassword_reset_expired_message'))
        self.assertEqual(
            selector.one('#django_cradmin_resetpassword_reset_expired_message').alltext_normalized,
            'This password reset link has expired.')

    def test_post_invalid_token(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser)
        response = self.client.post(self.__get_url('invalid-token'), {
            'password1': 'secret',
            'password2': 'secret',
        })
        selector = htmls.S(response.content)
        self.assertFalse(selector.exists('form#django_cradmin_resetpassword_reset_form'))
        self.assertTrue(selector.exists('#django_cradmin_resetpassword_reset_invalid_token_message'))
        self.assertEqual(
            selector.one('#django_cradmin_resetpassword_reset_invalid_token_message').alltext_normalized,
            'Invalid password reset URL. Are you sure you copied the entire URL from the email?')

    def test_post(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser)

        with self.settings(
                DJANGO_CRADMIN_RESETPASSWORD_FINISHED_REDIRECT_URL='http://example.com'):
            response = self.client.post(self.__get_url('valid-token'), {
                'password1': 'newpassword',
                'password2': 'newpassword',
            })
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], 'http://example.com')
        testuser = get_user_model().objects.get(pk=self.testuser.pk)
        self.assertTrue(testuser.check_password('newpassword'))

    def test_post_success_message(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser)

        request = RequestFactory().post('/test', {
            'password1': 'newpassword',
            'password2': 'newpassword',
        })
        request.user = self.testuser
        request._messages = mock.MagicMock()

        with self.settings(DJANGO_CRADMIN_RESETPASSWORD_FINISHED_REDIRECT_URL='http://example.com'):
            ResetPasswordView.as_view()(request, token='valid-token')
        request._messages.add.assert_called_once_with(
            messages.SUCCESS, 'Your password has been updated.', '')



from django.conf.urls import patterns, url

from django_cradmin.apps.cradmin_email.views import email_design

urlpatterns = patterns(
    '',
    url(r'^emaildesign/(?P<format>html|plaintext)?$', email_design.EmailDesignView.as_view()),
)



from django_cradmin.superuserui.views import mixins
from django_cradmin.viewhelpers import create


class View(mixins.QuerySetForRoleMixin, create.CreateView):
    enable_modelchoicefield_support = True



from django.conf import settings


class ImageTypeMapSettingNotDefined(Exception):
    """
    Raised when :meth:`.Interface.transform_image_using_imagetype` is called
    when the ``DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP`` setting is not defined.
    """


class InvalidImageType(Exception):
    """
    Raised when :meth:`.Interface.transform_image_using_imagetype` is called
    with an ``imagetype`` not in the ``DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP``
    setting.
    """


class Interface(object):
    """
    Imageutils backend interface.

    An object of subclass of this interface is loaded via the
    ``DJANGO_CRADMIN_IMAGEUTILS_BACKEND`` setting, and the object
    is a singleton that only has one instance for the entire
    Django server process.
    """
    def transform_image(self, imageurl, **options):
        """
        Transforms the given ``imageurl`` by applying the
        given options and returns the URL of the transformed image.

        All backends must support the following options:

        - ``width``: The width to scale the image to.
        - ``height``: The height to scale the image to.
        - ``quality``: The quality to transform the image to
          as an int between 0 and 100 (100 means 100% quality).
          Should default to 100.
        - ``crop``: Specifies how to crop the image. All backends must
          support the following values:

          - ``limit`` (default): The image should be scaled to not exceed the specified
            width and/or height while retaining original aspect ratio of the image.
          - ``fill``: The image should be cropped to fill the specified ``width`` and ``height``
            while retaining original proportions of the image.

        This method must be overridden for each backend.
        """
        raise NotImplementedError()

    def transform_image_using_imagetype(self, imageurl, imagetype):
        """
        Works just like :meth:`.transform_image`, except that the
        options is retrieved from the :setting:`DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP` setting.

        Raises :exc:`.ImageTypeMapSettingNotDefined` if the :setting:`DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP`
        setting is not defined.

        Raises :exc:`.InvalidImageType` if the ``imagetype`` key is not defined in
        the :setting:`DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP` setting.
        """
        if not hasattr(settings, 'DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP'):
            raise ImageTypeMapSettingNotDefined('The DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP setting is not defined.')
        try:
            options = settings.DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP[imagetype]
        except KeyError:
            raise InvalidImageType('The requested imagetype, {}, is not specified in the '
                                   'DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP setting.'.format(imagetype))
        return self.transform_image(imageurl, **options)



from __future__ import unicode_literals
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager
from django.db import models


class EmailUser(AbstractBaseUser):
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []
    objects = BaseUserManager()

    email = models.EmailField(
        max_length=250, blank=False,
        unique=True)

    def get_short_name(self):
        return self.email

    def get_full_name(self):
        return self.email



from __future__ import unicode_literals
from datetime import timedelta, datetime
from django.contrib import messages

from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.test import TestCase, RequestFactory
from django.utils import timezone
import htmls
from django_cradmin.python2_compatibility import mock
from django_cradmin.apps.cradmin_activate_account.views.activate import ActivateAccountView

from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata
from django_cradmin.tests.helpers import create_user


class TestActivateAccountView(TestCase):
    def setUp(self):
        self.testuser = create_user('testuser', is_active=False)

    def __get_url(self, token):
        return reverse('cradmin-activate-account-activate', kwargs={'token': token})

    def _create_generic_token_with_metadata(self, user, created_datetime=None, expiration_datetime=None,
                                            metadata=None, **kwargs):
        generic_token_with_metadata = GenericTokenWithMetadata(
            created_datetime=(created_datetime or timezone.now()),
            expiration_datetime=(expiration_datetime or (timezone.now() + timedelta(days=2))),
            app='cradmin_activate_account',
            content_object=user,
            **kwargs)
        if metadata:
            generic_token_with_metadata.metadata = metadata
        generic_token_with_metadata.save()
        return generic_token_with_metadata

    def test_get_expired_token(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser,
            expiration_datetime=datetime(2014, 1, 1))
        response = self.client.get(self.__get_url('valid-token'))
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists('#django_cradmin_activate_account_expired_message'))
        self.assertEqual(
            selector.one('#django_cradmin_activate_account_expired_message').alltext_normalized,
            'This account activation link has expired.')

    def test_get_invalid_token(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser)
        response = self.client.get(self.__get_url('invalid-token'))
        selector = htmls.S(response.content)
        self.assertTrue(selector.exists('#django_cradmin_activate_account_invalid_token_message'))
        self.assertEqual(
            selector.one('#django_cradmin_activate_account_invalid_token_message').alltext_normalized,
            'Invalid account activation URL. Are you sure you copied the entire URL from the email?')

    def test_get_redirect_ok(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser,
            metadata={'next_url': '/next'})
        response = self.client.get(self.__get_url('valid-token'))
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], 'http://testserver/next')

    def test_get_activate_ok(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser,
            metadata={'next_url': '/next'})
        self.assertFalse(self.testuser.is_active)
        self.client.get(self.__get_url('valid-token'))
        testuser = get_user_model().objects.get(id=self.testuser.id)
        self.assertTrue(testuser.is_active)

    def test_post_success_message(self):
        self._create_generic_token_with_metadata(
            token='valid-token', user=self.testuser,
            metadata={'next_url': '/next'})
        request = RequestFactory().get('/test')
        request.user = self.testuser
        request._messages = mock.MagicMock()
        ActivateAccountView.as_view()(request, token='valid-token')
        request._messages.add.assert_called_once_with(
            messages.SUCCESS, 'Your account is now active.', '')



from django_cradmin.viewhelpers.listfilter.basefilters.multi import abstractcheckbox
from django_cradmin.viewhelpers.listfilter.django.base import DjangoOrmFilterMixin


class RelatedModelOrFilter(abstractcheckbox.AbstractCheckboxFilter, DjangoOrmFilterMixin):
    """
    Filter on values from a related model (one-to-many or many-to-many).

    Allows the user to select the values from a list of checkboxes --- the values
    are defined by you. The query is an OR query, meaning that each selected
    value potentially increases the number of items shown instead of limiting it.

    You just have to override
    :meth:`~django_cradmin.viewhelpers.listfilter.basefilters.multi.abstractcheckbox.AbstractCheckboxFilter.get_choices`
    and :meth:`get_filter_attribute`.

    Examples:

        One-to-many relationship::

            class Person(models.Model):
                pass

            class Tag(models.Model):
                person = models.ForeignKey(Person, related_name='tags')
                tag = models.SlugField()

            class TagFilter(listfilter.django.multi.checkbox.RelatedModel):
                def get_choices(self):
                    return [
                        (tag, tag)
                        for tag in Tag.objects.values_list('tag', flat=True).distinct()
                    ]

                def get_filter_attribute(self):
                    return 'tags__tag'

        Many-to-many relationship::

            class Tag(models.Model):
                tag = models.SlugField()

            class Person(models.Model):
                tags = models.ManyToManyField(Tag)

            class TagFilter(listfilter.django.multi.checkbox.RelatedModel):
                def get_choices(self):
                    return [
                        (tag, tag)
                        for tag in Tag.objects.values_list('tag', flat=True).distinct()
                    ]

                def get_filter_attribute(self):
                    return 'tags__tag'

        These examples are fairly simple. You may want to adjust get_choices() to
        only get tags actually in use or by some other query, but this should
        be a good starting point.
    """
    def get_filter_attribute(self):
        """
        Get the attribute to filter on.

        Lets say you have a ``Person`` model and a Tag model with a
        foreignkey from Tag to Person named ``person`` with ``related_name="tags"``.
        If you want to filter your Person model by tag, you would return ``tags__tag``
        (assuming ``tag`` is the attribute you want to filter by).

        The same goes for many-to-many relationships, but there you
        would call your ManyToManyField ``tags``, and still return the same value.
        """
        raise NotImplementedError()

    def filter(self, queryobject):
        cleaned_values = self.get_cleaned_values()
        if cleaned_values:
            queryobject = queryobject.filter(
                **{'{}__in'.format(self.get_filter_attribute()): cleaned_values})
        return queryobject



from __future__ import unicode_literals

from builtins import str

from django import forms
from django.utils.translation import pgettext

from django_cradmin.viewhelpers.multiselect2 import manytomanywidget
from django_cradmin.widgets import datetimepicker
from django_cradmin.widgets import filewidgets
from django_cradmin.widgets import modelchoice


class ModelForm(forms.ModelForm):
    """
    A subclass of :class:`django.form.ModelForm` that automatically sets
    up good default widgets and other things for the fields.
    """
    def __init__(self, *args, **kwargs):
        """
        Parameters:
            view: The View object where this form is used. If this is
                provided, we are able to setup more widgets because
                some widgets depend on data from the view (such
                as ``view.request``).
        """
        self.view = kwargs.pop('view', None)
        super(ModelForm, self).__init__(*args, **kwargs)
        self.autosetup_fields()

    def setup_datetime_field(self, fieldname, formfield):
        """
        Called by :meth:`.setup_field` if the ``formfield`` is a DateTimeField.

        Sets up :class:`django_cradmin.widgets.datetimepicker.DateTimePickerWidget`
        as the widget.

        Parameters:
            fieldname: The name of the field.
            formfield: The form field object.
        """
        self.fields[fieldname].widget = datetimepicker.DateTimePickerWidget(
            required=formfield.required
        )

    def setup_date_field(self, fieldname, formfield):
        """
        Called by :meth:`.setup_field` if the ``formfield`` is a DateField.

        Sets up :class:`django_cradmin.widgets.datetimepicker.DatePickerWidget`
        as the widget.

        Parameters:
            fieldname: The name of the field.
            formfield: The form field object.
        """
        self.fields[fieldname].widget = datetimepicker.DatePickerWidget(
            required=formfield.required
        )

    def setup_file_field(self, fieldname, formfield):
        """
        Called by :meth:`.setup_field` if the ``formfield`` is a FileField.

        Sets up :class:`django_cradmin.widgets.filewidgets.FileWidget`
        as the widget.

        Parameters:
            fieldname: The name of the field.
            formfield: The form field object.
        """
        self.fields[fieldname].widget = filewidgets.FileWidget(
            clearable=not formfield.required
        )

    def setup_image_field(self, fieldname, formfield):
        """
        Called by :meth:`.setup_field` if the ``formfield`` is an ImageField.

        Sets up :class:`django_cradmin.widgets.filewidgets.ImageWidget`
        as the widget if the ``view``-argument is provided to the constructor
        of this form, falls back to :meth:`.setup_file_field` if not.

        Parameters:
            fieldname: The name of the field.
            formfield: The form field object.
        """
        if self.view:
            self.fields[fieldname].widget = filewidgets.ImageWidget(
                request=self.view.request,
                clearable=not formfield.required,
            )
        else:
            self.setup_file_field(fieldname=fieldname, formfield=formfield)

    def make_related_object_preview(self, fieldname, formfield, relatedobject):
        """
        Make preview for related object.

        Used by :meth:`.setup_modelchoice_field` to generate the preview for an
        object.

        Defaults to ``str(relatedobject)`` with fallback to ``(Not selected)`` (translatable)
        if ``relatedobject`` is ``None``.

        Parameters:
            fieldname: The name of the field.
            formfield: The form field object.
            relatedobject: The related model object.
        """
        if relatedobject is None:
            return pgettext('automodelform modelchoice_field no value', '(Not selected)')
        else:
            return str(relatedobject)

    def setup_modelchoice_field(self, fieldname, formfield):
        """
        Called by :meth:`.setup_field` if the ``formfield`` is a ModelChoiceField.

        Sets up :class:`django_cradmin.widgets.modelchoice.ModelChoiceWidget`
        as the widget if the ``view``-argument is provided to the constructor
        of this form, and if ``view.request.cradmin_instance.get_foreignkeyselectview_url`
        returns a view URL for the model in the ModelChoiceField.

        Parameters:
            fieldname: The name of the field.
            formfield: The form field object.
        """
        if not self.view or not hasattr(self.view.request, 'cradmin_instance'):
            return
        model_class = formfield.queryset.model
        cradmin_instance = self.view.request.cradmin_instance
        foreignkeyselectview_url = cradmin_instance.get_foreignkeyselectview_url(model_class=model_class)
        if not foreignkeyselectview_url:
            return

        preview = ''
        if self.instance:
            try:
                relatedobject = getattr(self.instance, fieldname)
            except model_class.DoesNotExist:
                pass
            else:
                preview = self.make_related_object_preview(fieldname=fieldname,
                                                           formfield=formfield,
                                                           relatedobject=relatedobject)

        self.fields[fieldname].widget = modelchoice.ModelChoiceWidget(
            queryset=formfield.queryset,
            preview=preview,
            selectview_url=foreignkeyselectview_url)

    def setup_modelmultichoice_field(self, fieldname, formfield):
        """
        Called by :meth:`.setup_field` if the ``formfield`` is a ModelMultiChoiceField.

        Sets up :class:`django_cradmin.viewhelpers.multiselect2.manytomanywidget.Widget`
        as the widget if the ``view``-argument is provided to the constructor
        of this form, and if ``view.request.cradmin_instance.get_manytomanyselectview_url`
        returns a view URL for the model in the ModelMultiChoiceField.

        Parameters:
            fieldname: The name of the field.
            formfield: The form field object.
        """
        if not self.view or not hasattr(self.view.request, 'cradmin_instance'):
            return
        model_class = formfield.queryset.model
        cradmin_instance = self.view.request.cradmin_instance
        manytomanyselectview_url = cradmin_instance.get_manytomanyselectview_url(model_class=model_class)
        if not manytomanyselectview_url:
            return

        self.fields[fieldname].widget = manytomanywidget.Widget(
            queryset=formfield.queryset,
            selectview_url=manytomanyselectview_url,
            required=formfield.required)

    def setup_field(self, fieldname, formfield):
        """
        Setup widgets and other properties from the given field.
        Called once for each field in the form by :meth:`.autosetup_fields`.

        We provide defaults for many field types:

        - :meth:`.setup_datetime_field`
        - :meth:`.setup_date_field`

        You can override this method to custom handle of some
        or all of your fields.

        Parameters:
            fieldname: The name of the field.
            formfield: The form field object.
        """
        if isinstance(formfield, forms.DateField):
            self.setup_date_field(fieldname=fieldname, formfield=formfield)
        elif isinstance(formfield, forms.DateTimeField):
            self.setup_datetime_field(fieldname=fieldname, formfield=formfield)
        elif isinstance(formfield, forms.ImageField):
            self.setup_image_field(fieldname=fieldname, formfield=formfield)
        elif isinstance(formfield, forms.FileField):
            self.setup_file_field(fieldname=fieldname, formfield=formfield)
        elif isinstance(formfield, forms.ModelMultipleChoiceField):
            self.setup_modelmultichoice_field(fieldname=fieldname, formfield=formfield)
        elif isinstance(formfield, forms.ModelChoiceField):
            self.setup_modelchoice_field(fieldname=fieldname, formfield=formfield)

    def autosetup_fields(self):
        """
        Automatically setup sane defaults for the fields in the form.
        """
        for fieldname, formfield in self.fields.items():
            self.setup_field(fieldname=fieldname, formfield=formfield)



import htmls
import mock
from django import test
from django.http import QueryDict

from django_cradmin.viewhelpers import listbuilder
from django_cradmin.viewhelpers.multiselect2 import manytomanyview


class TestViewMixin(test.TestCase):

    def test_get_selected_values_list(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.GET = QueryDict('', mutable=True)
        view.request.GET.update({
            'manytomany_select_current_value': '[1,2]'
        })
        self.assertEqual(
            [1, 2],
            view.get_selected_values_list())

    def test_get_selected_objects(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.cradmin_role = 'testrole'
        view.get_selected_values_list = mock.MagicMock(return_value='testvalue')
        mockqueryset = mock.MagicMock()
        view.get_queryset_for_role = mock.MagicMock(return_value=mockqueryset)

        view.get_selected_objects()
        mockqueryset.filter.assert_called_once_with(pk__in='testvalue')
        view.get_queryset_for_role.assert_called_once_with('testrole')

    def test_get_selected_objects_has_get_unfiltered_queryset_for_role_method(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.cradmin_role = 'testrole'
        view.get_selected_values_list = mock.MagicMock(return_value='testvalue')
        view.get_queryset_for_role = mock.MagicMock()
        mockqueryset = mock.MagicMock()
        view.get_unfiltered_queryset_for_role = mock.MagicMock(return_value=mockqueryset)

        view.get_selected_objects()
        mockqueryset.filter.assert_called_once_with(pk__in='testvalue')
        view.get_unfiltered_queryset_for_role.assert_called_once_with('testrole')
        view.get_queryset_for_role.assert_not_called()

    def test_should_include_previously_selected_no_paging_no_get_filters_string(self):
        view = manytomanyview.ViewMixin()
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_get_filters_string_empty_string(self):
        view = manytomanyview.ViewMixin()
        view.get_filters_string = mock.MagicMock(return_value='')
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_get_filters_string_nonempty_string(self):
        view = manytomanyview.ViewMixin()
        view.get_filters_string = mock.MagicMock(return_value='test')
        self.assertFalse(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_no_page_specified(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = 'page'
        view.kwargs = {}
        view.request = mock.MagicMock()
        view.request.GET = {}
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_page1_kwargs(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = 'page'
        view.kwargs = {'page': 1}
        view.request = mock.MagicMock()
        view.request.GET = {}
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_page1_requestget(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = 'page'
        view.kwargs = {}
        view.request = mock.MagicMock()
        view.request.GET = {'page': 1}
        self.assertTrue(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_page2_kwargs(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = 'page'
        view.kwargs = {'page': 2}
        view.request = mock.MagicMock()
        view.request.GET = {}
        self.assertFalse(view.should_include_previously_selected())

    def test_should_include_previously_selected_paginate_by_page2_requestget(self):
        view = manytomanyview.ViewMixin()
        view.paginate_by = 40
        view.page_kwarg = 'page'
        view.kwargs = {}
        view.request = mock.MagicMock()
        view.request.GET = {'page': 2}
        self.assertFalse(view.should_include_previously_selected())

    def test_get_queryset_for_role_should_include_previously_selected_false(self):
        mockqueryset = mock.MagicMock()

        class SuperMyView(object):
            def get_queryset_for_role(self, role):
                return mockqueryset

        class MyView(manytomanyview.ViewMixin, SuperMyView):
            pass

        view = MyView()
        view.request = mock.MagicMock()
        view.should_include_previously_selected = mock.MagicMock(return_value=False)
        view.get_queryset_for_role()
        mockqueryset.exclude.assert_not_called()

    def test_get_queryset_for_role_should_include_previously_selected_true(self):
        mockqueryset = mock.MagicMock()

        class SuperMyView(object):
            def get_queryset_for_role(self, role):
                return mockqueryset

        class MyView(manytomanyview.ViewMixin, SuperMyView):
            pass

        view = MyView()
        view.request = mock.MagicMock()
        view.should_include_previously_selected = mock.MagicMock(return_value=True)
        view.get_selected_values_list = mock.MagicMock(return_value='testvalue')
        view.get_queryset_for_role()
        mockqueryset.exclude.called_once_with(pk__in='testvalue')

    def test_get_target_renderer_class(self):
        view = manytomanyview.ViewMixin()
        self.assertEqual(
            manytomanyview.ViewMixin.target_renderer_class,
            view.get_target_renderer_class())

    def test_get_target_renderer_kwargs(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.GET = {'manytomany_select_fieldid': 'testid',
                            'manytomany_select_required': 'True'}
        target_renderer_kwargs = view.get_target_renderer_kwargs()
        self.assertEqual('testid', target_renderer_kwargs['target_formfield_id'])
        self.assertFalse(target_renderer_kwargs['empty_selection_allowed'])

    def test_get_target_renderer(self):
        view = manytomanyview.ViewMixin()
        view.request = mock.MagicMock()
        view.request.GET = {'manytomany_select_fieldid': 'testid',
                            'manytomany_select_required': 'True'}
        target_renderer = view.get_target_renderer()
        self.assertEqual('testid', target_renderer.target_formfield_id)


class TestListBuilderViewMixin(test.TestCase):

    def test_get_listbuilder_list_should_include_previously_selected_false(self):
        class SuperMyView(object):
            def get_listbuilder_list(self, context):
                return listbuilder.base.List()

        class MyView(manytomanyview.ListBuilderViewMixin, SuperMyView):
            pass

        view = MyView()
        view.should_include_previously_selected = mock.MagicMock(return_value=False)
        self.assertFalse(view.get_listbuilder_list(context={}).has_items())

    def test_get_listbuilder_list_should_include_previously_selected_true(self):
        class SuperMyView(object):
            def get_listbuilder_list(self, context):
                return listbuilder.base.List()

        class MyView(manytomanyview.ListBuilderViewMixin, SuperMyView):
            pass

        view = MyView()
        view.should_include_previously_selected = mock.MagicMock(return_value=True)
        view.get_value_renderer_class = mock.MagicMock(return_value=listbuilder.base.ItemValueRenderer)
        view.get_frame_renderer_class = mock.MagicMock(return_value=listbuilder.base.ItemFrameRenderer)
        view.get_selected_objects = mock.MagicMock(return_value=['testvalue'])
        selector = htmls.S(view.get_listbuilder_list(context={}).render())
        self.assertTrue('testvalue',
                        selector.one('.django-cradmin-listbuilder-itemvalue').alltext_normalized)


class ListBuilderFilterListViewMixin(test.TestCase):
    def test_get_filterlist_url(self):
        view = manytomanyview.ListBuilderFilterListViewMixin()
        view.request = mock.MagicMock()
        view.get_filterlist_url(filters_string='test')
        view.request.cradmin_app.reverse_appurl.assert_called_once_with(
            'manytomanyselect-filter', kwargs={'filters_string': 'test'})

    def test_add_target_renderer_to_filterlist(self):
        view = manytomanyview.ListBuilderFilterListViewMixin()
        view.get_target_renderer = mock.MagicMock(return_value='testrenderer')
        mockfilterlist = mock.MagicMock()
        view.add_target_renderer_to_filterlist(filterlist=mockfilterlist)
        mockfilterlist.append.assert_called_once_with('testrenderer')

    def test_add_filterlist_items(self):
        class SuperMyView(object):
            def add_filterlist_items(self, filterlist):
                pass

        class MyView(manytomanyview.ListBuilderFilterListViewMixin, SuperMyView):
            pass

        view = MyView()
        view.add_target_renderer_to_filterlist = mock.MagicMock()
        view.add_filterlist_items('testfilterlist')
        view.add_target_renderer_to_filterlist.assert_called_once_with(filterlist='testfilterlist')



from django.contrib import messages


class MessagesDebugMiddleware(object):
    """
    Add ``django_cradmin.messages_debug_middleware.MessagesDebugMiddleware``
    to your MIDDLEWARE_CLASSES setting to debug Django messages
    rendering/styling. Will add one of each message type to the request.
    """
    def process_request(self, request):
        messages.debug(request, 'A debug message.')
        messages.info(request, 'An info message.')
        messages.success(request, 'A success  message.')
        messages.warning(request, 'A warning message.')
        messages.error(request, 'An error message.')



from __future__ import unicode_literals
from builtins import object
from django.contrib import auth
from django.contrib.auth import models as auth_models


class EmailAuthBackend(object):
    """
    Custom Authentication backend for using `email` as your login-field on any ``User``-model. This will also work with
    the default django ``User``-model, as it does not require ``USERNAME_FIELD`` to be ``email``.
    """

    def authenticate(self, email, password):
        """
        Find the `User` corresponding to ``email``, verify ``password`` and return user.

        NOTE: this function is defined by Django as required for an Auth-backend

        :param email: ``email`` for the user to authenticate
        :param password: ``password`` for the user to authenticate
        :return: the ``User`` if authentication was successful, or ``None`` if not.
        """
        user = self.__get_user_from_email(email)
        if not user:
            return None

        password_valid = auth_models.check_password(password, user.password)

        if not password_valid:
            return None

        return user

    def get_user(self, user_id):
        """
        locate and return a :class:`User` based on the ``primary key`` of the current :class:`User` model.

        NOTE: this function is defined by Django as required for an Auth-backend

        :param user_id: the ``id`` of a :class:`User`
        :return: the ``User`` matching the given ``user_id`` if it exists, ``None`` if not.
        """
        try:
            return auth.get_user_model().objects.get(pk=user_id)
        except auth.get_user_model().DoesNotExist:
            return None

    def __get_user_from_email(self, email):
        """
        locate and return a :class:`User` based on the ``email``-field of the current :class:`User` model.

        this is simply a utility for ``authenticate()``

        :param email: the ``email``-field of a ``User``
        :return: the ``User`` matching the given ``email`` if it exists, ``None`` if not.
        """
        user_model = auth.get_user_model()
        try:
            return user_model.objects.get(email=email)
        except user_model.DoesNotExist:
            return None



from __future__ import unicode_literals
from django.conf.urls import url, patterns
from django_cradmin.apps.cradmin_activate_account.views.activate import ActivateAccountView

urlpatterns = patterns(
    '',
    url(r'^activate/(?P<token>.+)',
        ActivateAccountView.as_view(),
        name="cradmin-activate-account-activate"),
)



from __future__ import unicode_literals
from builtins import object
import logging

from django import http
from django.contrib import messages
from django.core.files.base import ContentFile
from django.utils.translation import ugettext_lazy as _
from crispy_forms import layout
from django import forms
from django.views.generic.edit import FormMixin

from django_cradmin.apps.cradmin_temporaryfileuploadstore.models import TemporaryFileCollection
from django_cradmin.apps.cradmin_temporaryfileuploadstore.widgets import BulkFileUploadWidget, SingleFileUploadWidget
from django_cradmin.utils import crhumanize
from django_cradmin.viewhelpers import objecttable
from django_cradmin.viewhelpers import crudbase
from django_cradmin.viewhelpers import update
from django_cradmin.viewhelpers import create
from django_cradmin.viewhelpers import delete
from django_cradmin.viewhelpers import formbase
from django_cradmin import crapp
from django_cradmin import crsettings
from django_cradmin.apps.cradmin_imagearchive.models import ArchiveImage

logger = logging.getLogger(__name__)


class DescriptionColumn(objecttable.MultiActionColumn):
    modelfield = 'description'

    def render_value(self, obj):
        return obj.screenreader_text

    def get_buttons(self, obj):
        edit_description_label = _('Set a description')
        if obj.description:
            edit_description_label = _('Edit description')
        return [
            objecttable.Button(
                label=edit_description_label,
                url=self.reverse_appurl('edit', args=[obj.id])),
            objecttable.Button(
                label=_('Delete'),
                url=self.reverse_appurl('delete', args=[obj.id]),
                buttonclass="btn btn-danger btn-sm"),
        ]


class DescriptionSelectColumn(objecttable.UseThisActionColumn):
    modelfield = 'description'

    def render_value(self, obj):
        return obj.screenreader_text

    def get_buttons(self, obj):
        return [
            objecttable.UseThisButton(
                view=self.view,
                label=_('Use this'),
                obj=obj)
        ]


class ImageColumn(objecttable.ImagePreviewColumn):
    modelfield = 'image'
    column_width = None

    preview_fallbackoptions = {
        'width': 100,
        'height': 65,
    }

    def get_column_width(self):
        if self.column_width:
            return self.column_width
        else:
            width = crsettings.get_setting('DJANGO_CRADMIN_IMAGEARCHIVE_LISTING_IMAGEWIDTH', 100)
            return '{}px'.format(width)

    def get_preview_imagetype(self):
        imagetype_from_settings = crsettings.get_setting('DJANGO_CRADMIN_IMAGEARCHIVE_LISTING_IMAGETYPE')
        if self.preview_imagetype:
            return self.preview_imagetype
        elif imagetype_from_settings:
            return imagetype_from_settings
        else:
            return None

    def get_header(self):
        return _('Preview')


class ArchiveImagesQuerySetForRoleMixin(object):
    """
    Used by listing, update and delete view to ensure
    that only images that the current role has access to
    is available.
    """
    def get_queryset_for_role(self, role):
        return ArchiveImage.objects.filter_owned_by_role(role)\
            .order_by('-created_datetime')


def get_bulkupload_apiparameters():
    apiparameters = {
        'accept': 'image/png,image/jpeg,image/gif',
    }
    max_filesize_bytes = crsettings.get_setting('DJANGO_CRADMIN_IMAGEARCHIVE_MAX_FILESIZE', None)
    if max_filesize_bytes is not None:
        apiparameters['max_filesize_bytes'] = crhumanize.dehumanize_readable_filesize(max_filesize_bytes)
    return apiparameters


class BulkAddForm(forms.Form):
    filecollectionid = forms.IntegerField(
        required=True,
        widget=BulkFileUploadWidget(
            autosubmit=True,
            accept='image/*',
            # accept='image/png,image/jpeg,image/gif',  # NOTE: Does not work with the fileselector in firefox
            apiparameters=get_bulkupload_apiparameters(),
            dropbox_text=_('Upload images by dragging and dropping them here'),
            invalid_filetype_message=_('Invalid filetype. You can only upload images.'),
            advanced_fileselectbutton_text=_('... or select images'),
            simple_fileselectbutton_text=_('Select images ...')
        ),
        label=_('Upload at least one image'),
        error_messages={
            'required': _('You must upload at least one image.')
        })


class SingleAddForm(forms.Form):
    filecollectionid = forms.IntegerField(
        required=True,
        widget=SingleFileUploadWidget(
            autosubmit=True,
            accept='image/*',
            apiparameters=get_bulkupload_apiparameters(),
            dropbox_text=_('Upload an image by dragging and dropping it here'),
            advanced_fileselectbutton_text=_('... or select an image'),
            invalid_filetype_message=_('Invalid filetype. You can only upload images.'),
            simple_fileselectbutton_text=_('Select an image ...')
        ),
        label=_('Upload an image'),
        error_messages={
            'required': _('You must upload an image.')
        })

    def clean(self):
        cleaned_data = super(SingleAddForm, self).clean()
        filecollectionid = cleaned_data.get('filecollectionid', None)
        if filecollectionid is not None:
            collection = TemporaryFileCollection.objects.get(id=filecollectionid)
            filecount = collection.files.count()
            if filecount < 1:
                raise forms.ValidationError({
                    'filecollectionid': _('You must upload an image.')
                })
            elif filecount > 1:
                raise forms.ValidationError({
                    'filecollectionid': _('You must upload exactly one image.')
                })


class AddImageOverlayButton(objecttable.NonSubmitButton):
    def get_html_attributes(self):
        attributes = super(AddImageOverlayButton, self).get_html_attributes()
        attributes['django-cradmin-bulkfileupload-show-overlay'] = 'filecollectionid'
        return attributes


class BaseImagesListView(ArchiveImagesQuerySetForRoleMixin, objecttable.ObjectTableView,
                         formbase.FormViewMixin, FormMixin):
    searchfields = ['name', 'description', 'file_extension']
    hide_column_headers = True
    model = ArchiveImage
    paginate_by = 15

    form_id = 'django_cradmin_imagearchive_bulkadd_form'
    extra_form_css_classes = ['django-cradmin-form-noasterisk', 'django-cradmin-bulkfileupload-form-overlay']
    template_name = 'django_cradmin/apps/cradmin_imagearchive/listview.django.html'

    def get_form_attributes(self):
        return {
            'django-cradmin-bulkfileupload-form': '',
            'django-cradmin-bulkfileupload-form-overlay': 'true',
            'django-cradmin-bulkfileupload-form-open-overlay-on-window-dragdrop': 'true'
        }

    def get_field_layout(self):
        return [
            layout.Div(
                'filecollectionid',
                # css_class="cradmin-globalfields"),
                css_class="cradmin-focusfield"),
        ]

    def get_buttons(self):
        return [
            AddImageOverlayButton(label=_('Add image'), buttonclass='btn btn-primary')
        ]

    def get_button_layout(self):
        # Overridden because get_buttons from formbase.FormView and
        # objecttable.ObjectTableView both use get_buttons().
        return []

    def get_formhelper(self):
        formhelper = super(BaseImagesListView, self).get_formhelper()
        formhelper.form_show_labels = False
        return formhelper

    def upload_file_to_archive(self, temporaryfile):
        archiveimage = ArchiveImage(
            role=self.request.cradmin_role,
            name=temporaryfile.filename)
        archiveimage.file_size = temporaryfile.file.size
        archiveimage.clean()
        archiveimage.save()
        archiveimage.image.save(temporaryfile.filename, ContentFile(temporaryfile.file.read()))
        archiveimage.full_clean()
        return archiveimage

    def upload_files_to_archive(self, temporaryfilecollection):
        uploaded_archiveimages = []
        for temporaryfile in temporaryfilecollection.files.all():
            uploaded_archiveimage = self.upload_file_to_archive(temporaryfile)
            uploaded_archiveimages.append(uploaded_archiveimage)
        return uploaded_archiveimages

    def get_collectionqueryset(self):
        return TemporaryFileCollection.objects\
            .filter_for_user(self.request.user)\
            .prefetch_related('files')

    def get_success_message(self, temporaryfilecollection):
        """
        Override this to provide a success message.

        The ``temporaryfilecollection`` is the TemporaryFileCollection that was just uploaded.

        Used by :meth:`.add_success_messages`.
        """
        filenames = [u'"{}"'.format(temporaryfile.filename) for temporaryfile in temporaryfilecollection.files.all()]
        return _(u'Uploaded %(what)s.') % {
            'what': u','.join(filenames)
        }

    def add_success_messages(self, temporaryfilecollection):
        """
        Add success messages on successful upload.

        The ``temporaryfilecollection`` is the TemporaryFileCollection that was just uploaded.

        Defaults to add :meth:`.get_success_message` as a django messages
        success message if :meth:`.get_success_message` returns anything.

        You can override this to add multiple messages or to show messages in some other way.
        """
        success_message = self.get_success_message(temporaryfilecollection)
        if success_message:
            messages.success(self.request, success_message)

    def form_valid(self, form):
        collectionid = form.cleaned_data['filecollectionid']
        try:
            temporaryfilecollection = self.get_collectionqueryset().get(id=collectionid)
        except TemporaryFileCollection.DoesNotExist:
            return http.HttpResponseNotFound()
        else:
            # We use ``uploaded_archiveimages`` in :meth:`.ArchiveImagesSingleSelectView.get_success_url`
            self.uploaded_archiveimages = self.upload_files_to_archive(temporaryfilecollection)
            self.add_success_messages(temporaryfilecollection)
            temporaryfilecollection.clear_files_and_delete()
            return http.HttpResponseRedirect(self.get_success_url())

    def form_invalid(self, form):
        self.object_list = self.get_queryset()
        return super(BaseImagesListView, self).form_invalid(form)

    def post(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)

    def get_context_data(self, **kwargs):
        context = super(BaseImagesListView, self).get_context_data(**kwargs)
        self.add_context_data(context)
        if 'form' not in context:
            form_class = self.get_form_class()
            form = self.get_form(form_class)
            context['form'] = form
        return context


class ArchiveImagesListView(BaseImagesListView):
    """
    The list view in the image achive app.
    """
    columns = [
        ImageColumn,
        DescriptionColumn,
    ]
    form_class = BulkAddForm

    # def get_button_layout(self):
    #     return [
    #         layout.Div(PrimarySubmit('save', _('Upload images')),
    #                    css_class="django_cradmin_submitrow")
    #     ]


class ArchiveImagesSingleSelectView(BaseImagesListView):
    """
    Used when selecting a single archive image as a foreign key.
    """
    columns = [
        ImageColumn,
        DescriptionSelectColumn,
    ]
    hide_menu = True
    form_class = SingleAddForm
    listing_viewname = 'singleselect'

    def make_foreignkey_preview_for(self, obj):
        archiveimage = obj
        return archiveimage.get_preview_html(request=self.request)

    # def get_button_layout(self):
    #     return [
    #         layout.Div(PrimarySubmit('save', _('Upload image')),
    #                    css_class="django_cradmin_submitrow")
    #     ]

    def get_success_url(self):
        url = self.request.build_absolute_uri()
        uploaded_archiveimage = self.uploaded_archiveimages[0]
        url = create.CreateView.add_foreignkey_selected_value_to_url_querystring(url, uploaded_archiveimage.pk)
        return url


class ArchiveImageUpdateView(crudbase.OnlySaveButtonMixin,
                             ArchiveImagesQuerySetForRoleMixin,
                             update.UpdateView):
    """
    View used to create edit existing images.
    """
    model = ArchiveImage
    fields = ['description']
    roleid_field = 'role'

    def get_field_layout(self):
        return [
            layout.Div(
                layout.Field('description', css_class='cradmin-textarea-small'),
                css_class='cradmin-globalfields'
            )
        ]


class ArchiveImageDeleteView(ArchiveImagesQuerySetForRoleMixin, delete.DeleteView):
    """
    View used to delete existing images.
    """


class App(crapp.App):
    appurls = [
        crapp.Url(
            r'^$',
            ArchiveImagesListView.as_view(),
            name=crapp.INDEXVIEW_NAME),
        crapp.Url(
            r'^singleselect$',
            ArchiveImagesSingleSelectView.as_view(),
            name='singleselect'),
        crapp.Url(
            r'^edit/(?P<pk>\d+)$',
            ArchiveImageUpdateView.as_view(),
            name="edit"),
        crapp.Url(
            r'^delete/(?P<pk>\d+)$',
            ArchiveImageDeleteView.as_view(),
            name="delete")
    ]



from __future__ import unicode_literals
from future import standard_library
standard_library.install_aliases()
from PIL import Image, ImageDraw
from io import BytesIO


def create_image(width, height, filetype='PNG'):
    """
    Create an image of the given ``width``, ``height`` and optional ``filetype``.

    Returns:
        A bytestring with the generated image.
    """
    im = Image.new('RGB', (width, height), color='#191919')
    ImageDraw.Draw(im)
    out = BytesIO()
    im.save(out, filetype)
    return out.getvalue()



from __future__ import unicode_literals
from builtins import next

from django.core import mail

from django.test import TestCase
from django_cradmin.python2_compatibility import mock

from django_cradmin.apps.cradmin_invite.invite_url import InviteUrl
from django_cradmin.tests.helpers import create_user


class InviteUrlMock(InviteUrl):
    def get_appname(self):
        return 'testapp'

    def get_confirm_invite_url(self, token):
        return '/invite/accept/{}'.format(token.token)


class InviteUrlWithStaticTokenMock(InviteUrlMock):
    def generate_generictoken(self, email=None):
        testtoken = mock.MagicMock()
        testtoken.token = 'test-token'
        return testtoken


class InviteUrlWithTokenIteratorMock(InviteUrlMock):
    def __init__(self, *args, **kwargs):
        self.tokens = iter(['token1', 'token2', 'token3'])
        super(InviteUrlWithTokenIteratorMock, self).__init__(*args, **kwargs)

    def _generate_generictoken(self, email=None):
        testtoken = mock.MagicMock()
        testtoken.token = next(self.tokens)
        return testtoken


class TestSendActivationEmail(TestCase):
    def setUp(self):
        self.testuser = create_user('testuser', email='testuser@example.com')

        # Any object will work as the target of invites.
        # We use a user object since we have that easily available
        self.invite_target = create_user('invitetarget')

    def test_generate_generictoken(self):
        token = InviteUrlMock(
            request=mock.MagicMock(),
            private=True,
            content_object=self.invite_target
        ).generate_generictoken()
        self.assertEquals(token.content_object, self.invite_target)

    def test_generate_generictoken_with_email(self):
        token = InviteUrlMock(
            request=mock.MagicMock(),
            private=True,
            content_object=self.invite_target
        ).generate_generictoken(email='test@example.com')
        self.assertEquals(token.metadata['email'], 'test@example.com')

    def test_generate_generictoken_with_email_and_metadata(self):
        token = InviteUrlMock(
            request=mock.MagicMock(),
            private=True,
            content_object=self.invite_target,
            metadata={'test': 10}
        ).generate_generictoken(email='test@example.com')
        self.assertEquals(token.metadata['email'], 'test@example.com')
        self.assertEquals(token.metadata['test'], 10)

    def test_generate_generictoken_with_email_and_metadata_with_email(self):
        token = InviteUrlMock(
            request=mock.MagicMock(),
            private=True,
            content_object=self.invite_target,
            metadata={'email': 'existing@example.com'}
        ).generate_generictoken(email='test@example.com')
        self.assertEquals(token.metadata['email'], 'existing@example.com')

    def test_send_email(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: 'http://testserver{}'.format(path)

        with self.settings(DJANGO_CRADMIN_SITENAME='Testsite'):
            InviteUrlWithStaticTokenMock(request=testrequest, content_object=self.invite_target, private=True)\
                .send_email('test@example.com')
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, 'Invitation to Testsite')
        self.assertIn('http://testserver/invite/accept/test-token',
                      mail.outbox[0].alternatives[0][0])
        self.assertIn('Click the button below to accept the invite.',
                      mail.outbox[0].alternatives[0][0])

    def test_send_email_private(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: 'http://testserver{}'.format(path)

        with self.settings(DJANGO_CRADMIN_SITENAME='Testsite'):
            inviteurl = InviteUrlWithTokenIteratorMock(
                request=testrequest, content_object=self.invite_target, private=True)
            inviteurl.send_email('test1@example.com', 'test2@example.com', 'test3@example.com')
        self.assertIn('token1', mail.outbox[0].body)
        self.assertIn('token2', mail.outbox[1].body)
        self.assertIn('token3', mail.outbox[2].body)

    def test_send_email_public(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: 'http://testserver{}'.format(path)

        with self.settings(DJANGO_CRADMIN_SITENAME='Testsite'):
            inviteurl = InviteUrlWithTokenIteratorMock(
                request=testrequest, content_object=self.invite_target, private=False)
            inviteurl.send_email('test1@example.com', 'test2@example.com', 'test3@example.com')
        self.assertIn('token1', mail.outbox[0].body)
        self.assertIn('token1', mail.outbox[1].body)
        self.assertIn('token1', mail.outbox[2].body)

    def test_get_share_url(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: 'http://testserver{}'.format(path)
        self.assertEqual(
            InviteUrlWithStaticTokenMock(
                request=testrequest, private=True, content_object=self.invite_target).get_share_url(),
            'http://testserver/invite/accept/test-token')

    def test_get_share_url_private(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: 'http://testserver{}'.format(path)
        inviteurl = InviteUrlWithTokenIteratorMock(
            request=testrequest, private=True, content_object=self.invite_target)
        self.assertEqual(inviteurl.get_share_url(), 'http://testserver/invite/accept/token1')
        self.assertEqual(inviteurl.get_share_url(), 'http://testserver/invite/accept/token2')
        self.assertEqual(inviteurl.get_share_url(), 'http://testserver/invite/accept/token3')

    def test_get_share_url_public(self):
        testrequest = mock.MagicMock()
        testrequest.build_absolute_uri = lambda path: 'http://testserver{}'.format(path)
        inviteurl = InviteUrlWithTokenIteratorMock(
            request=testrequest, private=False, content_object=self.invite_target)
        self.assertEqual(inviteurl.get_share_url(), 'http://testserver/invite/accept/token1')
        self.assertEqual(inviteurl.get_share_url(), 'http://testserver/invite/accept/token1')
        self.assertEqual(inviteurl.get_share_url(), 'http://testserver/invite/accept/token1')



from django import template
from django.template.loader import render_to_string

register = template.Library()


def _cradmin_email_buttonlink(parser, token, linkstyle_context_variable):
    try:
        tag_name, url = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires exactly one arguments" % token.contents.split()[0]
        )
    end_tag = 'end_{}'.format(tag_name)
    nodelist = parser.parse((end_tag,))
    parser.delete_first_token()
    return CradminEmailButtonlinkNode(nodelist=nodelist, url=url,
                                      linkstyle_context_variable=linkstyle_context_variable)


class CradminEmailButtonlinkNode(template.Node):
    def __init__(self, nodelist, url, linkstyle_context_variable):
        if url[0] in ('"', "'"):
            url = url[1:-1]
        else:
            # Assume we have a variable
            url = template.Variable(url)
        self.url = url
        self.nodelist = nodelist
        self.linkstyle_context_variable = linkstyle_context_variable

    def render(self, context):
        output = self.nodelist.render(context)

        if isinstance(self.url, template.Variable):
            url = self.url.resolve(context)
        else:
            url = self.url
        linkstyle = context.get(self.linkstyle_context_variable, '')
        return render_to_string('cradmin_email/templatetags/cradmin_email_buttonlink.django.html', {
            'url': url,
            'label': output,
            'linkstyle': linkstyle
        })


@register.tag
def cradmin_email_primary_buttonlink(parser, token):
    """
    Render a link as a primary button.

    Examples::

        Url as string:

        .. code-block:: htmldjango

            {% load cradmin_email_tags %}
            {% cradmin_email_primary_buttonlink "http://example.com" %}
                A primary button link
            {% end_cradmin_email_primary_buttonlink %}

        Url as context variable:

        .. code-block:: htmldjango

            {% load cradmin_email_tags %}
            {% cradmin_email_primary_buttonlink someurl %}
                A primary button link
            {% end_cradmin_email_primary_buttonlink %}
    """
    return _cradmin_email_buttonlink(parser, token,
                                     linkstyle_context_variable='primary_button_link_style')


@register.tag
def cradmin_email_secondary_buttonlink(parser, token):
    """
    Render a link as a secondary button.

    Examples::

        Url as string:

        .. code-block:: htmldjango

            {% load cradmin_email_tags %}
            {% cradmin_email_secondary_buttonlink "http://example.com" %}
                A secondary button link
            {% end_cradmin_email_secondary_buttonlink %}

        Url as context variable:

        .. code-block:: htmldjango

            {% load cradmin_email_tags %}
            {% cradmin_email_secondary_buttonlink someurl %}
                A secondary button link
            {% end_cradmin_email_primary_buttonlink %}
    """
    return _cradmin_email_buttonlink(parser, token,
                                     linkstyle_context_variable='secondary_button_link_style')



from __future__ import unicode_literals
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.contrib.auth import get_user_model
from django_cradmin.apps.cradmin_register_account.forms.base import AbstractCreateAccountWithPasswordForm


class AuthUserCreateAccountForm(AbstractCreateAccountWithPasswordForm):
    r"""
    A create account form for ``auth_user``.

    Can be used directly as the ``DJANGO_CRADMIN_REGISTER_ACCOUNT_FORM_CLASS``
    setting, or extended to create a custom register account form.

    To use it directly, set the following setting::

        DJANGO_CRADMIN_REGISTER_ACCOUNT_FORM_CLASS = \
            'django_cradmin.apps.cradmin_register_account.forms.auth_user.AuthUserCreateAccountForm'

    The form only includes username, email and password. To add more fields, simply
    override the Meta-class and the field layout. Lets say we want to make
    a form that includes the ``first_name`` and ``last_name`` fields::

        from django_cradmin.apps.cradmin_register_account.forms.auth_user import AuthUserCreateAccountForm

        class AuthUserCreateAccountWithFullNameForm(AuthUserCreateAccountForm):
            class Meta(AuthUserCreateAccountForm.Meta):
                fields = ['email', 'username', 'first_name', 'last_name']

            def get_field_layout(self):
                return [
                    'username',
                    'email',
                    'first_name',
                    'last_name',
                    'password1',
                    'password2',
                ]

    """
    class Meta(AbstractCreateAccountWithPasswordForm.Meta):
        fields = ['email', 'username']

    def __init__(self, *args, **kwargs):
        super(AuthUserCreateAccountForm, self).__init__(*args, **kwargs)
        self.fields['email'].required = True
        # self.fields['username'].required = True

    def clean_email(self):
        email = self.cleaned_data['email']
        user_model = get_user_model()
        if user_model.objects.filter(email=email).exists():
            raise forms.ValidationError(
                message=_('Account with this email address already exists.'),
                code='not_unique_email')
        return email

    def deactivate_user(self, user):
        user.is_active = False

    def get_field_layout(self):
        return [
            'username',
            'email',
            'password1',
            'password2',
        ]


class AuthUserCreateAccountAutoUsernameForm(AuthUserCreateAccountForm):
    r"""
    A create account form for ``auth_user`` that autocreates the username
    from the email.

    This is a subclass of :class:`.AuthUserCreateAccountForm`,
    and the examples for extending that class works for this class too.

    Can be used directly as the ``DJANGO_CRADMIN_REGISTER_ACCOUNT_FORM_CLASS``
    setting, or extended to create a custom register account form.

    To use it directly, set the following setting::

        DJANGO_CRADMIN_REGISTER_ACCOUNT_FORM_CLASS = \
            'django_cradmin.apps.cradmin_register_account.forms.auth_user.AuthUserCreateAccountAutoUsernameForm'

    """
    class Meta(AuthUserCreateAccountForm.Meta):
        fields = ['email']

    def set_username(self, user):
        user.username = user.email[0:30]

    def set_extra_user_attributes(self, user):
        self.set_username(user)

    def get_field_layout(self):
        return [
            'email',
            'password1',
            'password2',
        ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='GenericTokenWithMetadata',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('app', models.CharField(max_length=255, db_index=True)),
                ('token', models.CharField(unique=True, max_length=100)),
                ('created_datetime', models.DateTimeField(verbose_name='Created')),
                ('expiration_datetime', models.DateTimeField(default=None, null=True, verbose_name='Expires', blank=True)),
                ('single_use', models.BooleanField(default=True)),
                ('metadata_json', models.TextField(default='', blank=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django_cradmin.superuserui.views import mixins
from django_cradmin.viewhelpers import update


class View(mixins.QuerySetForRoleMixin, update.UpdateView):
    enable_modelchoicefield_support = True



import htmls
from django import test
from django.http import QueryDict

from django_cradmin import python2_compatibility
from django_cradmin.python2_compatibility import mock
from django_cradmin.viewhelpers.multiselect2 import manytomanywidget


class TestWidget(test.TestCase):
    def test_wrapper_div_id(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertTrue(selector.exists('#id_cradmin_modelmultichoicefieldwrapper_testname'))

    def test_wrapper_class(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertTrue(
            selector.one('#id_cradmin_modelmultichoicefieldwrapper_testname').hasclass(
                'django-cradmin-modelmultichoicefield-wrapper'))

    def test_wrapper_angularjs_directive_value(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=[1, 2], attrs={'id': 'testid'}))
        directive_value = selector.one('#id_cradmin_modelmultichoicefieldwrapper_testname')[
            'django-cradmin-model-choice-field-wrapper']
        path, querydictstring = directive_value.split('?')
        querydict = QueryDict(querydictstring)
        self.assertEqual('/test', path)
        self.assertEqual(
            {'manytomany_select_fieldid': 'testid',
             'manytomany_select_current_value': '[1, 2]',
             'manytomany_select_required': 'True'},
            querydict.dict())

    def test_inputfield_value_none(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertEqual(
            '[]',
            selector.one('input[name="testname"]')['value']
        )

    def test_inputfield_value_empty_string(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value='', attrs={'id': 'testid'}))
        self.assertEqual(
            '[]',
            selector.one('input[name="testname"]')['value']
        )

    def test_inputfield_value(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=[1, 2], attrs={'id': 'testid'}))
        self.assertEqual(
            '[1, 2]',
            selector.one('input[name="testname"]')['value']
        )

    def test_inputfield_type_input_field_visible_default(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertEqual(
            'hidden',
            selector.one('input[name="testname"]')['type']
        )

    def test_inputfield_type_input_field_visible_false(self):
        class MyWidget(manytomanywidget.Widget):
            input_field_visible = False

        mockqueryset = mock.MagicMock()
        selector = htmls.S(MyWidget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertEqual(
            'hidden',
            selector.one('input[name="testname"]')['type']
        )

    def test_inputfield_type_input_field_visible_true(self):
        class MyWidget(manytomanywidget.Widget):
            input_field_visible = True

        mockqueryset = mock.MagicMock()
        selector = htmls.S(MyWidget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertEqual(
            'text',
            selector.one('input[name="testname"]')['type']
        )

    def test_preview_list_sanity(self):
        mockqueryset = mock.MagicMock()
        mockqueryset.filter.return_value = []
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertTrue(selector.exists('.django-cradmin-multiselect2-preview-list'))

    def test_preview_list_angularjs_directive(self):
        mockqueryset = mock.MagicMock()
        mockqueryset.filter.return_value = []
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertTrue(
            selector.one('.django-cradmin-multiselect2-preview-list').hasattribute(
                'django-cradmin-model-choice-field-preview'))

    def test_preview_list_without_value(self):
        mockqueryset = mock.MagicMock()
        mockqueryset.filter.return_value = []
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertEqual(
            0,
            selector.count('.django-cradmin-multiselect2-preview-list li'))

    def test_preview_list_with_value(self):
        mockqueryset = mock.MagicMock()
        mockmodelobject = mock.MagicMock()
        mockmodelobject.pk = 1
        mockmodelobject.__str__.return_value = 'testvalue'
        if python2_compatibility.is_python2():
            mockmodelobject.__unicode__.return_value = 'testvalue'
        mockqueryset.filter.return_value = [mockmodelobject]
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertEqual(
            1,
            selector.count('.django-cradmin-multiselect2-preview-list li'))
        self.assertEqual(
            'testvalue',
            selector.one('.django-cradmin-multiselect2-preview-list '
                         '.django-cradmin-multiselect2-preview-list-value').alltext_normalized)

    def test_selectbutton_angularjs_directive(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertTrue(
            selector.one('button').hasattribute(
                'django-cradmin-model-choice-field-changebegin-button'))

    def test_selectbutton_css_class(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertEqual(
            'btn btn-default',
            selector.one('button')['class'])

    def test_selectbutton_text_default(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertEqual(
            'Select ...',
            selector.one('button').alltext_normalized)

    def test_selectbutton_text_custom(self):
        mockqueryset = mock.MagicMock()
        selector = htmls.S(manytomanywidget.Widget(
            queryset=mockqueryset,
            selectview_url='/test',
            selectbutton_text='Testing ...'
        ).render(name='testname', value=None, attrs={'id': 'testid'}))
        self.assertEqual(
            'Testing ...',
            selector.one('button').alltext_normalized)



import time

from django.conf import settings


class DelayMiddleware(object):
    """
    To use this, you must add the following to your settings:

    - Add ``django_cradmin.delay_middleware.DelayMiddleware`` to ``MIDDLEWARE_CLASSES``.
    - Set ``DJANGO_CRADMIN_DELAY_MIDDLEWARE_MILLISECONDS`` to the number of milliseconds
      delay you want to add to all requests (I.E.: 2000 for 2 seconds).
    """
    def process_request(self, request):
        time.sleep(settings.DJANGO_CRADMIN_DELAY_MIDDLEWARE_MILLISECONDS / 1000.0)
        return None



from __future__ import unicode_literals
from django_cradmin.viewhelpers import listfilter


class ViewMixin(object):
    """
    Mixin class for views using filterlist.

    See :class:`django_cradmin.viewhelpers.listbuilderview.FilterListMixin`
    and :class:`django_cradmin.viewhelpers.objecttable.FilterListMixin`
    for implementation examples.
    """
    #: The :class:`django_cradmin.viewhelpers.listfilter.base.abstractfilterlist.AbstractFilterList`
    #: subclass to use.
    #: Defaults to :class:`django_cradmin.viewhelpers.listfilter.lists.Vertical`.
    filterlist_class = listfilter.lists.Vertical

    def get_filterlist_url(self, filters_string):
        """
        Get the URL for the given ``filters_string``.

        This will typically involve reversing the current view
        URL and insert ``filters_string`` as the ``filters_string``
        kwarg.
        """
        raise NotImplementedError()

    def get_filter_unprotected_querystring_arguments(self):
        """
        Get a set of names querystring arguments/attributes
        that should be not be kep when a filter is applied.

        Lets say you store the current page as ``?page=2``.
        Then you would want to return ``{'p'}`` here make sure
        applying a filter returns the user to the first page.
        """
        return set()

    def filterlist_urlbuilder(self, filters_string):
        """
        Get an urlbuilder for :class:`django_cradmin.viewhelpers.listfilter.base.AbstractFilterList`.

        You should not override this. Override :meth:`.get_filterlist_url` instead.
        If you override this, you have to ensure you handle querystring correctly.
        """
        querydict = self.request.GET.copy()
        unprotected_querystring_arguments = self.get_filter_unprotected_querystring_arguments()
        if unprotected_querystring_arguments:
            for unprotected_querystring_argument in unprotected_querystring_arguments:
                if unprotected_querystring_argument in querydict:
                    del querydict[unprotected_querystring_argument]

        url = self.get_filterlist_url(filters_string)
        querystring = querydict.urlencode()
        if querystring:
            url = '{}?{}'.format(url, querystring)
        return url

    def get_filters_string(self):
        """
        Get the ``filters_string`` for
        :meth:`~django_cradmin.viewhelpers.listfilter.base.abstractfilterlist.AbstractFilterList.set_filters_string`.

        Defaults to ``kwargs.get('filters_string', '')``, which means that this just
        works as long as you use an URL pattern that looks something like this::

            r'^filter/(?P<filters_string>.+)?$'
        """
        return self.kwargs.get('filters_string', '')

    def get_filterlist_target_dom_id(self):
        """
        Get the DOM ID of the target element changed when the filters are changed.

        This element is dynamically updated by making a http request
        via javascript, looking up the target element
        (the element with the ID returned by this method) in the requested page,
        and replacing the current contents of the target element with the
        contents in the element returned dynamically.

        This means that the target element should be a wrapper around:

        - The content beeing filtered.
        - Anything that changes along with the filtered content (such as the pager)
          and any "no items message", "invalid input message" etc.

        Must be overridden in subclasses.
        """
        raise NotImplementedError()

    def get_filterlist_class(self):
        """
        Get the filterlist class.

        Should return a subclass of
        :class:`django_cradmin.viewhelpers.listfilter.base.abstractfilterlist.AbstractFilterList`.

        Defaults to :obj:`.ViewMixin.filterlist_class`.
        """
        return self.filterlist_class

    def get_label_is_screenreader_only_by_default(self):
        """
        Defines the default value of
        :meth:`.django_cradmin.viewhelpers.listfilter.base.abstractfilter.AbstractFilter.get_label_is_screenreader_only`
        for all filters in the filterlist.

        This is mostly useful if you want to not show labels in your filterlist.

        Defaults to ``False``.
        """
        return False

    def get_filterlist_kwargs(self):
        """
        Get kwargs for the :meth:`.get_filterlist_class`.

        Uses:

        - :meth:`.filterlist_urlbuilder` as the ``urlbuilder``-argument for the filterlist.
        - :meth:`.get_filterlist_target_dom_id` as the ``target_dom_id``-argument for the filterlist.
        - :meth:`.get_label_is_screenreader_only_by_default` as the
          ``label_is_screenreader_only_by_default`` argument for the filterlist.

        You should not need to override this unless you create
        a filterlist with extra parameters, since we have methods
        for overriding all the parameters for
        :class:`django_cradmin.viewhelpers.listfilter.base.abstractfilterlist.AbstractFilterList`.
        Override the methods listed above instead.

        Note that :meth:`.filterlist_urlbuilder` should
        not be overridden directly, but you should instead override
        :meth:`.get_filterlist_url` as documented in :meth:`.filterlist_urlbuilder`.
        """
        return {
            'urlbuilder': self.filterlist_urlbuilder,
            'target_dom_id': self.get_filterlist_target_dom_id(),
            'label_is_screenreader_only_by_default': self.get_label_is_screenreader_only_by_default()
        }

    def make_empty_filterlist(self):
        """
        Creates an empty filterlist.

        Uses:

        - :meth:`.get_filterlist_class` as the filterlist class.
        - :meth:`.get_filterlist_kwargs` as parameters when inializing the filterlist.

        You should not override this method, but instead override the
        methods above.
        """
        filterlist_class = self.get_filterlist_class()
        return filterlist_class(**self.get_filterlist_kwargs())

    def add_filterlist_items(self, filterlist):
        """
        Add items to the given ``filterlist``.

        The ``filterlist`` is created by :meth:`.make_empty_filterlist`
        by :meth:`.build_filterlist`.

        Parameters:
            filterlist: An object of a subclass of
                :class:`django_cradmin.viewhelpers.listfilter.base.AbstractFilterList`.
        """
        pass

    def build_filterlist(self):
        """
        Build the filterlist.

        Creates an empty filterlist with :meth:`.make_empty_filterlist`,
        then adds filters and other items with :meth:`.add_filterlist_items`,
        and sets the filters string using :meth:`.get_filters_string`.
        """
        filterlist = self.make_empty_filterlist()
        self.add_filterlist_items(filterlist=filterlist)
        filterlist.set_filters_string(filters_string=self.get_filters_string())
        return filterlist

    def get_filterlist(self):
        """
        Returns the result of :meth:`.build_filterlist`, but
        ensures the method is only called once for a request.

        This means that you should override :meth:`.build_filterlist`,
        but when you need to use it, you should call this method instead.
        """
        if not hasattr(self, '_filterlist'):
            self._filterlist = self.build_filterlist()
        return self._filterlist

    def get_filterlist_position(self):
        """
        Get the position where you want to place the filterlist.

        The supported values are defined in the mixins implemented
        in the various modules that support filterlist
        (such as :class:`django_cradmin.viewhelpers.listbuilderview.FilterListMixin`).
        """
        raise NotImplementedError()

    def get_filterlist_template_name(self):
        """
        Get the template to use based on what :meth:`.get_filterlist_position`.

        You will want to call this from the ``get_template_names`` method.
        This is just the interface, refer to the mixins implemented in
        various modules (such as :class:`django_cradmin.viewhelpers.listbuilderview.FilterListMixin`)
        for details on how to use this method.
        """
        raise NotImplementedError()

    def get_template_names(self):
        """
        You should not override this - override :meth:`.get_filterlist_template_name`
        instead.
        """
        return [self.get_filterlist_template_name()]

    def add_filterlist_to_context_data(self, context_data):
        """
        Add the context data required to render the filterlist.
        You may need to use this if you have some strange inheritance
        where get_context_data() is not called for all superclasses
        of your view.
        """
        context_data['filterlist'] = self.get_filterlist()

    def get_context_data(self, **kwargs):
        context = super(ViewMixin, self).get_context_data(**kwargs)
        context['filterlist'] = self.get_filterlist()
        return context

    def get_unfiltered_queryset_for_role(self, role):
        """
        Get a queryset with all objects of :obj:`~.ViewMixin.model`  that
        the current role can access.
        """
        raise NotImplementedError()

    def get_queryset_for_role(self, role):
        """
        Uses :meth:`.get_unfiltered_queryset_for_role` ot get the base
        queryset, then we apply the filters using
        ``get_filterlist.filter(queryset)``.

        You should not need to override this - override :meth:`.get_unfiltered_queryset_for_role`
        instead.
        """
        queryset = self.get_unfiltered_queryset_for_role(role)
        queryset = self.get_filterlist().filter(queryset)
        return queryset.distinct()



import htmls
from django import test

from django_cradmin.viewhelpers.multiselect2 import widget_preview_renderer


class TestValue(test.TestCase):
    def test_css_class_added(self):
        selector = htmls.S(widget_preview_renderer.Value(
            value='testvalue').render())
        self.assertTrue(selector.exists('.django-cradmin-multiselect2-preview-list-value'))

    def test_wrap_in_li_element_false(self):
        selector = htmls.S(widget_preview_renderer.Value(
            value='testvalue', wrap_in_li_element=False).render())
        self.assertFalse(selector.exists('li'))

    def test_wrap_in_li_element_true(self):
        selector = htmls.S(widget_preview_renderer.Value(
            value='testvalue', wrap_in_li_element=True).render())
        self.assertTrue(selector.exists('li'))

    def test_render_value_wrap_in_li_element_false(self):
        selector = htmls.S(widget_preview_renderer.Value(
            value='testvalue', wrap_in_li_element=False).render())
        self.assertEqual(
            'testvalue',
            selector.one('.django-cradmin-listbuilder-itemvalue').alltext_normalized)

    def test_render_value_wrap_in_li_element_true(self):
        selector = htmls.S(widget_preview_renderer.Value(
            value='testvalue', wrap_in_li_element=True).render())
        self.assertEqual(
            'testvalue',
            selector.one('.django-cradmin-listbuilder-itemvalue').alltext_normalized)


class TestList(test.TestCase):
    def test_css_class_added(self):
        selector = htmls.S(widget_preview_renderer.List().render())
        self.assertTrue(selector.one('ol').hasclass('django-cradmin-multiselect2-preview-list'))

    def test_from_value_iterable_uses_the_new_default_value_render(self):
        listbuilder_list = widget_preview_renderer.List.from_value_iterable(
            value_iterable=['testvalue'])
        selector = htmls.S(listbuilder_list.render())
        self.assertTrue(selector.exists('.django-cradmin-multiselect2-preview-list-value'))



from __future__ import unicode_literals
from builtins import range
from django import test

from django_cradmin.tests.sortable.cradmin_sortable_testapp.models import SortableItem
from django_cradmin.tests.sortable.cradmin_sortable_testapp.models import ItemContainer


class TestSortableItem(test.TestCase):
    """
    Test that classes inherit :class:`SortableBase` works as advertised!
    """

    def setUp(self):
        pass

    def _create_container(self):
        container = ItemContainer(name='test container')
        container.save()
        return container

    def _create_items(self, num_items, container=None):
        container = container or self._create_container()
        items = []
        for i in range(0, num_items, 1):
            items.append(SortableItem(
                container=container,
                name='test{}'.format(i),
                sort_index=i))
        SortableItem.objects.bulk_create(items)
        return SortableItem.objects.all()

    def test_has_sort_index_field(self):
        item = self._create_items(1)[0]
        self.assertTrue(hasattr(item, 'sort_index'))
        self.assertEquals(item.sort_index, 0)

    def test_sort_item_last(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_last(i1)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [i2.id, i3.id, i4.id, i5.id, i6.id, i1.id])

    def test_sort_item_3_last(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_last(i3)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [i1.id, i2.id, i4.id, i5.id, i6.id, i3.id])

    def test_sort_last_item_first(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_before(i6, i1.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [i6.id, i1.id, i2.id, i3.id, i4.id, i5.id])

    def test_sort_item_3_first(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_before(i3, i1.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [i3.id, i1.id, i2.id, i4.id, i5.id, i6.id])

    def test_sort_item_move_first_before_id_4(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_before(i1, i4.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [i2.id, i3.id, i1.id, i4.id, i5.id, i6.id])

    def test_sort_item_move_second_before_id_4(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_before(i2, i4.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [i1.id, i3.id, i2.id, i4.id, i5.id, i6.id])

    def test_sort_item_move_second_before_id_4_in_allready_reordered_list(self):
        i1, i2, i3, i4, i5, i6 = self._create_items(6)
        SortableItem.objects.sort_last(i1)
        # now list is [i2,i3,i4,i5,i6,i1]
        SortableItem.objects.sort_before(i2, i4.id)
        reordered = [si.id for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(reordered, [i3.id, i2.id, i4.id, i5.id, i6.id, i1.id])

    def test_set_sort_index_to_last_no_siblings(self):
        container = self._create_container()
        item = SortableItem(container=container, name='My test')
        SortableItem.objects.set_newitem_sort_index_to_last(item)
        self.assertEquals(item.sort_index, 0)
        self.assertIsNone(item.pk)  # The item was not saved

    def test_set_sort_index_to_last_has_siblings(self):
        container = self._create_container()
        self._create_items(2, container=container)
        item = SortableItem(container=container, name='My test')
        SortableItem.objects.set_newitem_sort_index_to_last(item)
        self.assertEquals(item.sort_index, 2)
        self.assertIsNone(item.pk)  # The item was not saved

    def test_set_sort_index_to_last_refuse_with_sort_index(self):
        container = self._create_container()
        item = SortableItem(container=container, name='My test', sort_index=1)
        with self.assertRaises(ValueError):
            SortableItem.objects.set_newitem_sort_index_to_last(item)

    def test_set_sort_index_to_last_refuse_existing(self):
        container = self._create_container()
        item = SortableItem(container=container, name='My test', sort_index=1)
        item.save()
        item.sort_index = None
        with self.assertRaises(ValueError):
            SortableItem.objects.set_newitem_sort_index_to_last(item)

    #####
    # Additional tests to locate bug where sort_before gives value of 0
    #####

    def test_sort_before_two_objects_last_to_first_not_lessthan_zero(self):
        i1, i2 = self._create_items(2)
        SortableItem.objects.sort_before(i2, i1.id)

        for object in SortableItem.objects.all():
            self.assertTrue(object.sort_index >= 0)

    def test_sort_before_three_objects_last_to_first_not_lessthan_zero(self):
        i1, i2, i3 = self._create_items(3)
        SortableItem.objects.sort_before(i3, i1.id)

        for object in SortableItem.objects.all():
            self.assertTrue(object.sort_index >= 0)

    def test_sort_before_four_objects_last_to_first_not_lessthan_zero(self):
        i1, i2, i3, i4 = self._create_items(4)
        SortableItem.objects.sort_before(i4, i1.id)

        for object in SortableItem.objects.all():
            self.assertTrue(object.sort_index >= 0)

    def test_sort_before_two_objects_last_to_second_not_lessthan_zero(self):
        i1, i2 = self._create_items(2)
        SortableItem.objects.sort_before(i1, i1.id)

        for object in SortableItem.objects.all():
            self.assertTrue(object.sort_index >= 0)

    def test_sort_before_three_objects_last_to_second_not_lessthan_zero(self):
        i1, i2, i3 = self._create_items(3)
        SortableItem.objects.sort_before(i3, i2.id)

        for object in SortableItem.objects.all():
            self.assertTrue(object.sort_index >= 0)

    def test_sort_before_four_objects_last_to_second_not_lessthan_zero(self):
        i1, i2, i3, i4 = self._create_items(4)
        SortableItem.objects.sort_before(i4, i2.id)

        for object in SortableItem.objects.all():
            self.assertTrue(object.sort_index >= 0)

    def test_sort_before_two_objects_first_to_last_not_lessthan_zero(self):
        i1, i2 = self._create_items(2)
        SortableItem.objects.sort_before(i1, i2.id)

        for object in SortableItem.objects.all():
            self.assertTrue(object.sort_index >= 0)

    def test_sort_before_three_objects_first_to_last_not_lessthan_zero(self):
        i1, i2, i3 = self._create_items(3)
        SortableItem.objects.sort_before(i1, i3.id)

        for object in SortableItem.objects.all():
            self.assertTrue(object.sort_index >= 0)

    def test_sort_before_four_objects_first_to_last_not_lessthan_zero(self):
        i1, i2, i3, i4 = self._create_items(4)
        SortableItem.objects.sort_before(i1, i4.id)

        for object in SortableItem.objects.all():
            self.assertTrue(object.sort_index >= 0)


class TestRepairSortable(test.TestCase):
    def setUp(self):
        pass

    def _create_container(self):
        container = ItemContainer(name='test container')
        container.save()
        return container

    def _create_items(self, num_items, container=None, first_sort_index=0):
        container = container or self._create_container()
        items = []
        for i in range(first_sort_index, num_items):
            items.append(self._create_item(container, i))
        return items

    def _create_item(self, container, sort_index):
        si = SortableItem.objects.create(
            container=container,
            name='test{}'.format(sort_index),
            sort_index=sort_index)
        return si

    def test_repairs_empty_hole(self):
        container = self._create_container()
        items = self._create_items(2, container)
        items.append(self._create_item(container, items[1].sort_index + 2))

        SortableItem.objects.sort_before(items[1], items[0].id)

        indexes = [si.sort_index for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(indexes, [0, 1, 2])

    def test_repairs_duplicate_index(self):
        container = self._create_container()
        items = self._create_items(2, container)
        items.append(self._create_item(container, items[1].sort_index))

        SortableItem.objects.sort_before(items[1], items[0].id)

        indexes = [si.sort_index for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(indexes, [0, 1, 2])

    def test_repair_multiple_duplicate_indexes(self):
        container = self._create_container()
        items = self._create_items(2, container)
        items.append(self._create_item(container, items[1].sort_index))
        items.append(self._create_item(container, items[1].sort_index + 1))
        items.append(self._create_item(container, items[1].sort_index + 2))
        items.append(self._create_item(container, items[1].sort_index + 2))

        SortableItem.objects.sort_before(items[1], items[0].id)

        indexes = [si.sort_index for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(indexes, [0, 1, 2, 3, 4, 5])

    def test_repair_multiple_holes_in_indexes(self):
        container = self._create_container()
        items = self._create_items(2, container)
        items.append(self._create_item(container, items[1].sort_index + 2))
        items.append(self._create_item(container, items[1].sort_index + 3))
        items.append(self._create_item(container, items[1].sort_index + 5))
        items.append(self._create_item(container, items[1].sort_index + 7))

        SortableItem.objects.sort_before(items[1], items[0].id)

        indexes = [si.sort_index for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(indexes, [0, 1, 2, 3, 4, 5])

    def test_repair_holes_and_duplicates_in_indexes(self):
        container = self._create_container()
        items = self._create_items(2, container)
        items.append(self._create_item(container, items[1].sort_index))
        items.append(self._create_item(container, items[1].sort_index + 2))
        items.append(self._create_item(container, items[1].sort_index + 3))

        SortableItem.objects.sort_before(items[1], items[0].id)

        indexes = [si.sort_index for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(indexes, [0, 1, 2, 3, 4])

    def test_repair_several_indexes_hole(self):
        container = self._create_container()
        items = self._create_items(2, container)
        items.append(self._create_item(container, items[1].sort_index + 3))

        SortableItem.objects.sort_before(items[1], items[0].id)

        indexes = [si.sort_index for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(indexes, [0, 1, 2])

    def test_repair_hole_at_beginning(self):
        container = self._create_container()
        i1 = self._create_item(container, 1)
        i2 = self._create_item(container, 2)

        SortableItem.objects.sort_before(i2, i1.id)
        indexes = [si.sort_index for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(indexes, [0, 1])

    def test_repair_duplicate_at_beginning(self):
        container = self._create_container()
        i1 = self._create_item(container, 0)
        i2 = self._create_item(container, 0)
        self._create_item(container, 1)

        SortableItem.objects.sort_before(i2, i1.id)
        indexes = [si.sort_index for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(indexes, [0, 1, 2])

    def test_repair_duplicate_at_end(self):
        container = self._create_container()
        i1 = self._create_item(container, 0)
        i2 = self._create_item(container, 1)
        self._create_item(container, 1)

        SortableItem.objects.sort_before(i2, i1.id)
        indexes = [si.sort_index for si in SortableItem.objects.all().order_by('sort_index')]
        self.assertEquals(indexes, [0, 1, 2])



from builtins import str

from django.utils.translation import pgettext_lazy

from django_cradmin import renderable


class SelectedItem(renderable.AbstractRenderableWithCss):
    """
    Renders a selected item. This is rendered by
    :class:`django_cradmin.viewhelpers.multiselect2.listbuilder_itemvalues.ItemValue`,
    and added to a :class:`django_cradmin.viewhelpers.multiselect2.target_renderer.Target`
    when the "select"-button in the ItemValue is clicked.
    """

    #: The template used to render this selected item.
    template_name = 'django_cradmin/viewhelpers/multiselect2/selected_item_renderer/selected-item.django.html'

    #: If this is specified, we will add an attribute with this name
    #: for the value as an attribute of the object.
    #:
    #: I.e.: if ``valuealias = "person"``, you will be able to use ``me.person`` in
    #: the template, and you will be able to use ``self.person`` in any methods you
    #: add or override in the class (just remember to call ``super()`` if you override
    #: ``__init__``).
    valuealias = None

    def __init__(self, value):
        """
        Args:
            value: The value to render. Typically a django model object.

        """
        self.value = value
        if self.valuealias:
            setattr(self, self.valuealias, self.value)

    def get_base_css_classes_list(self):
        return ['django-cradmin-multiselect2-selected-item']

    def get_title(self):
        """
        Returns:
            str: The title of the selected item.
        """
        return str(self.value)

    def get_description(self):
        """
        Returns:
            str: The description of the selected item.

            This is shown below the title. It defaults to ``None``,
            which means that no description is rendered.
        """
        return None

    def get_deselectbutton_text(self):
        """
        Returns:
            str: The text for the deselect button.

            Defaults to the ``deselectbutton_text`` parameter for ``__init__``,
            falling back on ``"Deselect"`` (translatable).
        """
        return pgettext_lazy('multiselect2 deselect button', 'Deselect')

    def get_deselectbutton_aria_label(self):
        """
        Returns:
            str: The text for the aria-label of the deselect button.

            Defaults to ``"Deselect <self.get_title()>"`` (translatable).
        """
        return pgettext_lazy('multiselect2 deselect button', 'Deselect "%(title)s"') % {
            'title': self.get_title()
        }

    def get_deselect_button_base_cssclasses_list(self):
        """
        Get the list of base css classes to use for the deselect button.
        You normally do not want to override this. Override
        :meth:`.get_deselect_button_extra_cssclasses_list` instead.
        """
        return [
            'django-cradmin-multiselect2-target-selected-item-deselectbutton'
        ]

    def get_deselect_button_extra_cssclasses_list(self):
        """
        Get the list of extra css classes to use for the deselect button.

        Defaults to ``['btn', 'btn-default']``.
        Override this to provide your own css classes.
        """
        return [
            'btn',
            'btn-default',
        ]

    def get_deselect_button_cssclasses_list(self):
        """
        Get the list of css classes to use for the deselect button.

        You normally do not want to override this. Override
        :meth:`.get_deselect_button_extra_cssclasses_list` instead.
        """
        return [
            'btn',
            'btn-default',
            'django-cradmin-multiselect2-target-selected-item-deselectbutton'
        ]

    def get_deselect_button_cssclasses_string(self):
        css_classes = self.get_deselect_button_base_cssclasses_list() + \
                      self.get_deselect_button_extra_cssclasses_list()
        return ' '.join(css_classes)

    def get_inputfield_name(self):
        """
        Returns:
            str: The name of the input field.

            Defaults to the ``inputfield_name`` parameter for ``__init__``, falling
            back on ``selected_items``.
        """
        return 'selected_items'

    def get_inputfield_value(self):
        """
        Returns:
            str or int: The value to put in the ``value`` attribute of the input field.
        """
        return self.value.pk



from __future__ import unicode_literals

from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import UpdateView as DjangoUpdateView

from django_cradmin.crispylayouts import PrimarySubmit
from django_cradmin.crispylayouts import DefaultSubmit
from .crudbase import CreateUpdateViewMixin
from django_cradmin.viewhelpers.mixins import QuerysetForRoleMixin


class UpdateView(QuerysetForRoleMixin, CreateUpdateViewMixin, DjangoUpdateView):
    template_name = 'django_cradmin/viewhelpers/update.django.html'

    def get_pagetitle(self):
        """
        Get the page title (the title tag).

        Defaults to ``Edit <verbose_name model>``.
        """
        return _('Edit %(what)s') % {'what': self.get_model_class()._meta.verbose_name}

    def get_buttons(self):
        buttons = [
            PrimarySubmit(self.get_submit_save_button_name(), self.get_submit_save_label()),
            DefaultSubmit(self.get_submit_save_and_continue_edititing_button_name(),
                          self.get_submit_save_and_continue_edititing_label()),
        ]
        self.add_preview_button_if_configured(buttons)
        return buttons

    def get_formhelper(self):
        helper = super(UpdateView, self).get_formhelper()
        helper.form_id = 'django_cradmin_updateform'
        return helper

    def get_success_message(self, object):
        return _('Saved "%(object)s".') % {'object': object}


class UpdateRoleView(UpdateView):
    """
    Extends :class:`.UpdateView` to streamline editing the current role
    object.

    Just like :class:`.UpdateView`, but with the get_object and
    get_queryset_for_role methods implemented to edit the current role
    object.
    """
    def get_object(self, queryset=None):
        return self.get_queryset_for_role(self.request.cradmin_role).get()

    def get_queryset_for_role(self, role):
        return self.get_model_class().objects.filter(pk=role.pk)


class RedirectToCreateIfDoesNotExistMixin(object):
    """
    An update view mixin that redirects to a create view when the
    object requested does not exist.

    You will typically use this for objects with a OneToOne relationship
    to the current role, but that may not exist. Then you would use
    something like::

        class MyUpdateView(update.UpdateView, update.RedirectToCreateIfDoesNotExistMixin):
            def get_object(self, queryset=None):
                return self.get_queryset_for_role(self.request.cradmin_role).get()

            def get_queryset_for_role(self, role):
                return self.get_model_class().objects.filter(someonetooneattr=role)

    And the view will automatically redirect to the create view
    if the object does not exist.
    """

    #: The viewname within this app for the create view.
    #: See :meth:`.get_createurl`. Defaults to ``create``.
    createview_appurl_name = 'create'

    def get_createurl(self):
        """
        Get the URL of the create view that you want to redirect to if
        the requested object does not exist.

        Defaults to::

            self.request.cradmin_app.reverse_appurl(self.createview_appurl_name)
        """
        return self.request.cradmin_app.reverse_appurl(self.createview_appurl_name)

    def get(self, request, *args, **kwargs):
        try:
            return super().get(request, *args, **kwargs)
        except self.get_model_class().DoesNotExist:
            return HttpResponseRedirect(self.request.cradmin_app.reverse_appurl('create'))
